# coding: utf8
from __future__ import unicode_literals

from .tokenizer_exceptions import TOKENIZER_EXCEPTIONS
from .norm_exceptions import NORM_EXCEPTIONS
from .punctuation import TOKENIZER_INFIXES, TOKENIZER_SUFFIXES
from .stop_words import STOP_WORDS
from .lex_attrs import LEX_ATTRS
from .morph_rules import MORPH_RULES
from ..tag_map import TAG_MAP

from ..tokenizer_exceptions import BASE_EXCEPTIONS
from ..norm_exceptions import BASE_NORMS
from ...language import Language
from ...attrs import LANG, NORM
from ...util import update_exc, add_lookups


class DanishDefaults(Language.Defaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters.update(LEX_ATTRS)
    lex_attr_getters[LANG] = lambda text: 'da'
    lex_attr_getters[NORM] = add_lookups(Language.Defaults.lex_attr_getters[NORM],
                                         BASE_NORMS, NORM_EXCEPTIONS)
    tokenizer_exceptions = update_exc(BASE_EXCEPTIONS, TOKENIZER_EXCEPTIONS)
    morph_rules = MORPH_RULES
    infixes = TOKENIZER_INFIXES
    suffixes = TOKENIZER_SUFFIXES
    tag_map = TAG_MAP
    stop_words = STOP_WORDS


class Danish(Language):
    lang = 'da'
    Defaults = DanishDefaults


__all__ = ['Danish']



# coding: utf8
from __future__ import unicode_literals
from collections import defaultdict

cimport numpy as np
import numpy
import numpy.linalg
from libc.math cimport sqrt

from .doc cimport token_by_start, token_by_end, get_token_attr
from ..structs cimport TokenC, LexemeC
from ..typedefs cimport flags_t, attr_t, hash_t
from ..attrs cimport attr_id_t
from ..parts_of_speech cimport univ_pos_t
from ..util import normalize_slice
from ..attrs cimport IS_PUNCT, IS_SPACE
from ..lexeme cimport Lexeme
from ..compat import is_config
from .. import about
from .underscore import Underscore


cdef class Span:
    """A slice from a Doc object."""
    @classmethod
    def set_extension(cls, name, default=None, method=None,
                      getter=None, setter=None):
        Underscore.span_extensions[name] = (default, method, getter, setter)

    @classmethod
    def get_extension(cls, name):
        return Underscore.span_extensions.get(name)

    @classmethod
    def has_extension(cls, name):
        return name in Underscore.span_extensions

    def __cinit__(self, Doc doc, int start, int end, attr_t label=0,
                  vector=None, vector_norm=None):
        """Create a `Span` object from the slice `doc[start : end]`.

        doc (Doc): The parent document.
        start (int): The index of the first token of the span.
        end (int): The index of the first token after the span.
        label (uint64): A label to attach to the Span, e.g. for named entities.
        vector (ndarray[ndim=1, dtype='float32']): A meaning representation
            of the span.
        RETURNS (Span): The newly constructed object.
        """
        if not (0 <= start <= end <= len(doc)):
            raise IndexError

        self.doc = doc
        self.start = start
        self.start_char = self.doc[start].idx if start < self.doc.length else 0
        self.end = end
        if end >= 1:
            self.end_char = self.doc[end - 1].idx + len(self.doc[end - 1])
        else:
            self.end_char = 0
        assert label in doc.vocab.strings, label
        self.label = label
        self._vector = vector
        self._vector_norm = vector_norm

    def __richcmp__(self, Span other, int op):
        if other is None:
            if op == 0 or op == 1 or op == 2:
                return False
            else:
                return True
        # Eq
        if op == 0:
            return self.start_char < other.start_char
        elif op == 1:
            return self.start_char <= other.start_char
        elif op == 2:
            return self.start_char == other.start_char and self.end_char == other.end_char
        elif op == 3:
            return self.start_char != other.start_char or self.end_char != other.end_char
        elif op == 4:
            return self.start_char > other.start_char
        elif op == 5:
            return self.start_char >= other.start_char

    def __hash__(self):
        return hash((self.doc, self.label, self.start_char, self.end_char))

    def __len__(self):
        """Get the number of tokens in the span.

        RETURNS (int): The number of tokens in the span.
        """
        self._recalculate_indices()
        if self.end < self.start:
            return 0
        return self.end - self.start

    def __repr__(self):
        if is_config(python3=True):
            return self.text
        return self.text.encode('utf-8')

    def __getitem__(self, object i):
        """Get a `Token` or a `Span` object

        i (int or tuple): The index of the token within the span, or slice of
            the span to get.
        RETURNS (Token or Span): The token at `span[i]`.

        EXAMPLE:
            >>> span[0]
            >>> span[1:3]
        """
        self._recalculate_indices()
        if isinstance(i, slice):
            start, end = normalize_slice(len(self), i.start, i.stop, i.step)
            return Span(self.doc, start + self.start, end + self.start)
        else:
            if i < 0:
                return self.doc[self.end + i]
            else:
                return self.doc[self.start + i]

    def __iter__(self):
        """Iterate over `Token` objects.

        YIELDS (Token): A `Token` object.
        """
        self._recalculate_indices()
        for i in range(self.start, self.end):
            yield self.doc[i]

    @property
    def _(self):
        """User space for adding custom attribute extensions."""
        return Underscore(Underscore.span_extensions, self,
                          start=self.start_char, end=self.end_char)

    def as_doc(self):
        # TODO: fix
        """Create a `Doc` object view of the Span's data. This is mostly
        useful for C-typed interfaces.

        RETURNS (Doc): The `Doc` view of the span.
        """
        cdef Doc doc = Doc(self.doc.vocab)
        doc.length = self.end-self.start
        doc.c = &self.doc.c[self.start]
        doc.mem = self.doc.mem
        doc.is_parsed = self.doc.is_parsed
        doc.is_tagged = self.doc.is_tagged
        doc.noun_chunks_iterator = self.doc.noun_chunks_iterator
        doc.user_hooks = self.doc.user_hooks
        doc.user_span_hooks = self.doc.user_span_hooks
        doc.user_token_hooks = self.doc.user_token_hooks
        doc.vector = self.vector
        doc.vector_norm = self.vector_norm
        for key, value in self.doc.cats.items():
            if hasattr(key, '__len__') and len(key) == 3:
                cat_start, cat_end, cat_label = key
                if cat_start == self.start_char and cat_end == self.end_char:
                    doc.cats[cat_label] = value
        return doc

    def merge(self, *args, **attributes):
        """Retokenize the document, such that the span is merged into a single
        token.

        **attributes: Attributes to assign to the merged token. By default,
            attributes are inherited from the syntactic root token of the span.
        RETURNS (Token): The newly merged token.
        """
        return self.doc.merge(self.start_char, self.end_char, *args,
                              **attributes)

    def similarity(self, other):
        """Make a semantic similarity estimate. The default estimate is cosine
        similarity using an average of word vectors.

        other (object): The object to compare with. By default, accepts `Doc`,
            `Span`, `Token` and `Lexeme` objects.
        RETURNS (float): A scalar similarity score. Higher is more similar.
        """
        if 'similarity' in self.doc.user_span_hooks:
            self.doc.user_span_hooks['similarity'](self, other)
        if len(self) == 1 and hasattr(other, 'orth'):
            if self[0].orth == other.orth:
                return 1.0
        elif hasattr(other, '__len__') and len(self) == len(other):
            for i in range(len(self)):
                if self[i].orth != getattr(other[i], 'orth', None):
                    break
            else:
                return 1.0
        if self.vector_norm == 0.0 or other.vector_norm == 0.0:
            return 0.0
        return numpy.dot(self.vector, other.vector) / (self.vector_norm * other.vector_norm)

    def get_lca_matrix(self):
        """Calculates the lowest common ancestor matrix for a given `Span`.
        Returns LCA matrix containing the integer index of the ancestor, or -1
        if no common ancestor is found (ex if span excludes a necessary
        ancestor). Apologies about the recursion, but the impact on
        performance is negligible given the natural limitations on the depth
        of a typical human sentence.
        """
        def __pairwise_lca(token_j, token_k, lca_matrix, margins):
            offset = margins[0]
            token_k_head = token_k.head if token_k.head.i in range(*margins) else token_k
            token_j_head = token_j.head if token_j.head.i in range(*margins) else token_j
            token_j_i = token_j.i - offset
            token_k_i = token_k.i - offset
            if lca_matrix[token_j_i][token_k_i] != -2:
                return lca_matrix[token_j_i][token_k_i]
            elif token_j == token_k:
                lca_index = token_j_i
            elif token_k_head == token_j:
                lca_index = token_j_i
            elif token_j_head == token_k:
                lca_index = token_k_i
            elif (token_j_head == token_j) and (token_k_head == token_k):
                lca_index = -1
            else:
                lca_index = __pairwise_lca(token_j_head, token_k_head, lca_matrix, margins)
            lca_matrix[token_j_i][token_k_i] = lca_index
            lca_matrix[token_k_i][token_j_i] = lca_index
            return lca_index

        lca_matrix = numpy.empty((len(self), len(self)), dtype=numpy.int32)
        lca_matrix.fill(-2)
        margins = [self.start, self.end]
        for j in range(len(self)):
            token_j = self[j]
            for k in range(len(self)):
                token_k = self[k]
                lca_matrix[j][k] = __pairwise_lca(token_j, token_k, lca_matrix, margins)
                lca_matrix[k][j] = lca_matrix[j][k]
        return lca_matrix

    cpdef np.ndarray to_array(self, object py_attr_ids):
        """Given a list of M attribute IDs, export the tokens to a numpy
        `ndarray` of shape `(N, M)`, where `N` is the length of the document.
        The values will be 32-bit integers.

        attr_ids (list[int]): A list of attribute ID ints.
        RETURNS (numpy.ndarray[long, ndim=2]): A feature matrix, with one row
            per word, and one column per attribute indicated in the input
            `attr_ids`.
        """
        cdef int i, j
        cdef attr_id_t feature
        cdef np.ndarray[attr_t, ndim=2] output
        # Make an array from the attributes --- otherwise our inner loop is Python
        # dict iteration.
        cdef np.ndarray[attr_t, ndim=1] attr_ids = numpy.asarray(py_attr_ids, dtype=numpy.uint64)
        cdef int length = self.end - self.start
        output = numpy.ndarray(shape=(length, len(attr_ids)), dtype=numpy.uint64)
        for i in range(self.start, self.end):
            for j, feature in enumerate(attr_ids):
                output[i-self.start, j] = get_token_attr(&self.doc.c[i], feature)
        return output

    cpdef int _recalculate_indices(self) except -1:
        if self.end > self.doc.length \
        or self.doc.c[self.start].idx != self.start_char \
        or (self.doc.c[self.end-1].idx + self.doc.c[self.end-1].lex.length) != self.end_char:
            start = token_by_start(self.doc.c, self.doc.length, self.start_char)
            if self.start == -1:
                raise IndexError("Error calculating span: Can't find start")
            end = token_by_end(self.doc.c, self.doc.length, self.end_char)
            if end == -1:
                raise IndexError("Error calculating span: Can't find end")

            self.start = start
            self.end = end + 1

    property vocab:
        """RETURNS (Vocab): The Span's Doc's vocab."""
        def __get__(self):
            return self.doc.vocab

    property sent:
        """RETURNS (Span): The sentence span that the span is a part of."""
        def __get__(self):
            if 'sent' in self.doc.user_span_hooks:
                return self.doc.user_span_hooks['sent'](self)
            # This should raise if we're not parsed.
            self.doc.sents
            cdef int n = 0
            root = &self.doc.c[self.start]
            while root.head != 0:
                root += root.head
                n += 1
                if n >= self.doc.length:
                    raise RuntimeError
            return self.doc[root.l_edge:root.r_edge + 1]

    property has_vector:
        """RETURNS (bool): Whether a word vector is associated with the object.
        """
        def __get__(self):
            if 'has_vector' in self.doc.user_span_hooks:
                return self.doc.user_span_hooks['has_vector'](self)
            elif self.vocab.vectors.data.size > 0:
                return any(token.has_vector for token in self)
            elif self.doc.tensor.size > 0:
                return True
            else:
                return False

    property vector:
        """A real-valued meaning representation. Defaults to an average of the
        token vectors.

        RETURNS (numpy.ndarray[ndim=1, dtype='float32']): A 1D numpy array
            representing the span's semantics.
        """
        def __get__(self):
            if 'vector' in self.doc.user_span_hooks:
                return self.doc.user_span_hooks['vector'](self)
            if self._vector is None:
                self._vector = sum(t.vector for t in self) / len(self)
            return self._vector

    property vector_norm:
        """RETURNS (float): The L2 norm of the vector representation."""
        def __get__(self):
            if 'vector_norm' in self.doc.user_span_hooks:
                return self.doc.user_span_hooks['vector'](self)
            cdef float value
            cdef double norm = 0
            if self._vector_norm is None:
                norm = 0
                for value in self.vector:
                    norm += value * value
                self._vector_norm = sqrt(norm) if norm != 0 else 0
            return self._vector_norm

    property sentiment:
        """RETURNS (float): A scalar value indicating the positivity or
            negativity of the span.
        """
        def __get__(self):
            if 'sentiment' in self.doc.user_span_hooks:
                return self.doc.user_span_hooks['sentiment'](self)
            else:
                return sum([token.sentiment for token in self]) / len(self)

    property text:
        """RETURNS (unicode): The original verbatim text of the span."""
        def __get__(self):
            text = self.text_with_ws
            if self[-1].whitespace_:
                text = text[:-1]
            return text

    property text_with_ws:
        """The text content of the span with a trailing whitespace character if
        the last token has one.

        RETURNS (unicode): The text content of the span (with trailing
            whitespace).
        """
        def __get__(self):
            return u''.join([t.text_with_ws for t in self])

    property noun_chunks:
        """Yields base noun-phrase `Span` objects, if the document has been
        syntactically parsed. A base noun phrase, or "NP chunk", is a noun
        phrase that does not permit other NPs to be nested within it – so no
        NP-level coordination, no prepositional phrases, and no relative
        clauses.

        YIELDS (Span): Base noun-phrase `Span` objects
        """
        def __get__(self):
            if not self.doc.is_parsed:
                raise ValueError(
                    "noun_chunks requires the dependency parse, which "
                    "requires a statistical model to be installed and loaded. "
                    "For more info, see the "
                    "documentation: \n%s\n" % about.__docs_models__)
            # Accumulate the result before beginning to iterate over it. This
            # prevents the tokenisation from being changed out from under us
            # during the iteration. The tricky thing here is that Span accepts
            # its tokenisation changing, so it's okay once we have the Span
            # objects. See Issue #375
            spans = []
            cdef attr_t label
            for start, end, label in self.doc.noun_chunks_iterator(self):
                spans.append(Span(self.doc, start, end, label=label))
            for span in spans:
                yield span

    property root:
        """The token within the span that's highest in the parse tree.
        If there's a tie, the earliest is prefered.

        RETURNS (Token): The root token.

        EXAMPLE: The root token has the shortest path to the root of the
            sentence (or is the root itself). If multiple words are equally
            high in the tree, the first word is taken. For example:

            >>> toks = nlp(u'I like New York in Autumn.')

            Let's name the indices – easier than writing `toks[4]` etc.

            >>> i, like, new, york, in_, autumn, dot = range(len(toks))

            The head of 'new' is 'York', and the head of "York" is "like"

            >>> toks[new].head.text
            'York'
            >>> toks[york].head.text
            'like'

            Create a span for "New York". Its root is "York".

            >>> new_york = toks[new:york+1]
            >>> new_york.root.text
            'York'

            Here's a more complicated case, raised by issue #214:

            >>> toks = nlp(u'to, north and south carolina')
            >>> to, north, and_, south, carolina = toks
            >>> south.head.text, carolina.head.text
            ('north', 'to')

            Here "south" is a child of "north", which is a child of "carolina".
            Carolina is the root of the span:

            >>> south_carolina = toks[-2:]
            >>> south_carolina.root.text
            'carolina'
        """
        def __get__(self):
            self._recalculate_indices()
            if 'root' in self.doc.user_span_hooks:
                return self.doc.user_span_hooks['root'](self)
            # This should probably be called 'head', and the other one called
            # 'gov'. But we went with 'head' elsehwhere, and now we're stuck =/
            cdef int i
            # First, we scan through the Span, and check whether there's a word
            # with head==0, i.e. a sentence root. If so, we can return it. The
            # longer the span, the more likely it contains a sentence root, and
            # in this case we return in linear time.
            for i in range(self.start, self.end):
                if self.doc.c[i].head == 0:
                    return self.doc[i]
            # If we don't have a sentence root, we do something that's not so
            # algorithmically clever, but I think should be quite fast,
            # especially for short spans.
            # For each word, we count the path length, and arg min this measure.
            # We could use better tree logic to save steps here...But I
            # think this should be okay.
            cdef int current_best = self.doc.length
            cdef int root = -1
            for i in range(self.start, self.end):
                if self.start <= (i+self.doc.c[i].head) < self.end:
                    continue
                words_to_root = _count_words_to_root(&self.doc.c[i], self.doc.length)
                if words_to_root < current_best:
                    current_best = words_to_root
                    root = i
            if root == -1:
                return self.doc[self.start]
            else:
                return self.doc[root]

    property lefts:
        """ Tokens that are to the left of the span, whose head is within the
        `Span`.

        YIELDS (Token):A left-child of a token of the span.
        """
        def __get__(self):
            for token in reversed(self):  # Reverse, so we get tokens in order
                for left in token.lefts:
                    if left.i < self.start:
                        yield left

    property rights:
        """Tokens that are to the right of the Span, whose head is within the
        `Span`.

        YIELDS (Token): A right-child of a token of the span.
        """
        def __get__(self):
            for token in self:
                for right in token.rights:
                    if right.i >= self.end:
                        yield right

    property n_lefts:
        """RETURNS (int): The number of leftward immediate children of the
            span, in the syntactic dependency parse.
        """
        def __get__(self):
            return len(list(self.lefts))

    property n_rights:
        """RETURNS (int): The number of rightward immediate children of the
            span, in the syntactic dependency parse.
        """
        def __get__(self):
            return len(list(self.rights))

    property subtree:
        """Tokens that descend from tokens in the span, but fall outside it.

        YIELDS (Token): A descendant of a token within the span.
        """
        def __get__(self):
            for word in self.lefts:
                yield from word.subtree
            yield from self
            for word in self.rights:
                yield from word.subtree

    property ent_id:
        """RETURNS (uint64): The entity ID."""
        def __get__(self):
            return self.root.ent_id

        def __set__(self, hash_t key):
            raise NotImplementedError(
                "Can't yet set ent_id from Span. Vote for this feature on "
                "the issue tracker: http://github.com/explosion/spaCy/issues")

    property ent_id_:
        """RETURNS (unicode): The (string) entity ID."""
        def __get__(self):
            return self.root.ent_id_

        def __set__(self, hash_t key):
            raise NotImplementedError(
                "Can't yet set ent_id_ from Span. Vote for this feature on the "
                "issue tracker: http://github.com/explosion/spaCy/issues")

    property orth_:
        """Verbatim text content (identical to Span.text). Exists mostly for
        consistency with other attributes.

        RETURNS (unicode): The span's text."""
        def __get__(self):
            return self.text

    property lemma_:
        """RETURNS (unicode): The span's lemma."""
        def __get__(self):
            return ' '.join([t.lemma_ for t in self]).strip()

    property upper_:
        """Deprecated. Use Span.text.upper() instead."""
        def __get__(self):
            return ''.join([t.text_with_ws.upper() for t in self]).strip()

    property lower_:
        """Deprecated. Use Span.text.lower() instead."""
        def __get__(self):
            return ''.join([t.text_with_ws.lower() for t in self]).strip()

    property string:
        """Deprecated: Use Span.text_with_ws instead."""
        def __get__(self):
            return ''.join([t.text_with_ws for t in self])

    property label_:
        """RETURNS (unicode): The span's label."""
        def __get__(self):
            return self.doc.vocab.strings[self.label]


cdef int _count_words_to_root(const TokenC* token, int sent_length) except -1:
    # Don't allow spaces to be the root, if there are
    # better candidates
    if Lexeme.c_check_flag(token.lex, IS_SPACE) and token.l_kids == 0 and token.r_kids == 0:
        return sent_length-1
    if Lexeme.c_check_flag(token.lex, IS_PUNCT) and token.l_kids == 0 and token.r_kids == 0:
        return sent_length-1
    cdef int n = 0
    while token.head != 0:
        token += token.head
        n += 1
        if n >= sent_length:
            raise RuntimeError(
                "Array bounds exceeded while searching for root word. This "
                "likely means the parse tree is in an invalid state. Please "
                "report this issue here: "
                "http://github.com/explosion/spaCy/issues")
    return n



# coding: utf8
from __future__ import print_function
# NB! This breaks in plac on Python 2!!
# from __future__ import unicode_literals

if __name__ == '__main__':
    import plac
    import sys
    from spacy.cli import download, link, info, package, train, convert
    from spacy.cli import vocab, init_model, profile, evaluate, validate
    from spacy.util import prints

    commands = {
        'download': download,
        'link': link,
        'info': info,
        'train': train,
        'evaluate': evaluate,
        'convert': convert,
        'package': package,
        'vocab': vocab,
        'init-model': init_model,
        'profile': profile,
        'validate': validate
    }
    if len(sys.argv) == 1:
        prints(', '.join(commands), title="Available commands", exits=1)
    command = sys.argv.pop(1)
    sys.argv[0] = 'spacy %s' % command
    if command in commands:
        plac.call(commands[command], sys.argv[1:])
    else:
        prints(
            "Available: %s" % ', '.join(commands),
            title="Unknown command: %s" % command,
            exits=1)



# coding: utf8
from __future__ import unicode_literals

import gc

from ...lang.en import English


def test_issue1506():
    nlp = English()

    def string_generator():
        for _ in range(10001):
            yield u"It's sentence produced by that bug."

        for _ in range(10001):
            yield u"I erase some hbdsaj lemmas."

        for _ in range(10001):
            yield u"I erase lemmas."

        for _ in range(10001):
            yield u"It's sentence produced by that bug."

        for _ in range(10001):
            yield u"It's sentence produced by that bug."

    for i, d in enumerate(nlp.pipe(string_generator())):
        # We should run cleanup more than one time to actually cleanup data.
        # In first run — clean up only mark strings as «not hitted».
        if i == 10000 or i == 20000 or i == 30000:
            gc.collect()

        for t in d:
            str(t.lemma_)



# coding: utf8
from __future__ import unicode_literals


# setting explicit height and max-width: none on the SVG is required for
# Jupyter to render it properly in a cell

TPL_DEP_SVG = """
<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" id="{id}" class="displacy" width="{width}" height="{height}" style="max-width: none; height: {height}px; color: {color}; background: {bg}; font-family: {font}">{content}</svg>
"""


TPL_DEP_WORDS = """
<text class="displacy-token" fill="currentColor" text-anchor="middle" y="{y}">
    <tspan class="displacy-word" fill="currentColor" x="{x}">{text}</tspan>
    <tspan class="displacy-tag" dy="2em" fill="currentColor" x="{x}">{tag}</tspan>
</text>
"""


TPL_DEP_ARCS = """
<g class="displacy-arrow">
    <path class="displacy-arc" id="arrow-{id}-{i}" stroke-width="{stroke}px" d="{arc}" fill="none" stroke="currentColor"/>
    <text dy="1.25em" style="font-size: 0.8em; letter-spacing: 1px">
        <textPath xlink:href="#arrow-{id}-{i}" class="displacy-label" startOffset="50%" fill="currentColor" text-anchor="middle">{label}</textPath>
    </text>
    <path class="displacy-arrowhead" d="{head}" fill="currentColor"/>
</g>
"""


TPL_FIGURE = """
<figure style="margin-bottom: 6rem">{content}</figure>
"""

TPL_TITLE = """
<h2 style="margin: 0">{title}</h2>
"""


TPL_ENTS = """
<div class="entities" style="line-height: 2.5">{content}</div>
"""


TPL_ENT = """
<mark class="entity" style="background: {bg}; padding: 0.45em 0.6em; margin: 0 0.25em; line-height: 1; border-radius: 0.35em; box-decoration-break: clone; -webkit-box-decoration-break: clone">
    {text}
    <span style="font-size: 0.8em; font-weight: bold; line-height: 1; border-radius: 0.35em; text-transform: uppercase; vertical-align: middle; margin-left: 0.5rem">{label}</span>
</mark>
"""


TPL_PAGE = """
<!DOCTYPE html>
<html>
    <head>
        <title>displaCy</title>
    </head>

    <body style="font-size: 16px; font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol'; padding: 4rem 2rem;">{content}</body>
</html>
"""



# coding: utf8
from __future__ import unicode_literals
import pytest


@pytest.mark.models('en')
def test_issue1959(EN):
    texts = ['Apple is looking at buying U.K. startup for $1 billion.']
    # nlp = load_test_model('en_core_web_sm')
    EN.add_pipe(clean_component, name='cleaner', after='ner')
    doc = EN(texts[0])
    doc_pipe = [doc_pipe for doc_pipe in EN.pipe(texts)]
    assert doc == doc_pipe[0]


def clean_component(doc):
    """ Clean up text. Make lowercase and remove punctuation and stopwords """
    # Remove punctuation, symbols (#) and stopwords
    doc = [tok.text.lower() for tok in doc if (not tok.is_stop
                                               and tok.pos_ != 'PUNCT' and
                                               tok.pos_ != 'SYM')]
    doc = ' '.join(doc)
    return doc



# coding: utf-8
from __future__ import unicode_literals

from ...pipeline import EntityRecognizer
from ..util import get_doc

import pytest


def test_doc_add_entities_set_ents_iob(en_vocab):
    text = ["This", "is", "a", "lion"]
    doc = get_doc(en_vocab, text)
    ner = EntityRecognizer(en_vocab)
    ner.begin_training([])
    ner(doc)

    assert len(list(doc.ents)) == 0
    assert [w.ent_iob_ for w in doc] == (['O'] * len(doc))

    doc.ents = [(doc.vocab.strings['ANIMAL'], 3, 4)]
    assert [w.ent_iob_ for w in doc] == ['O', 'O', 'O', 'B']

    doc.ents = [(doc.vocab.strings['WORD'], 0, 2)]
    assert [w.ent_iob_ for w in doc] == ['B', 'I', 'O', 'O']



from __future__ import unicode_literals

from ...vocab import Vocab
from ...lang.lex_attrs import LEX_ATTRS
from ...tokens import Doc
from ...matcher import Matcher


def test_issue1434():
    '''Test matches occur when optional element at end of short doc'''
    vocab = Vocab(lex_attr_getters=LEX_ATTRS)
    hello_world = Doc(vocab, words=['Hello', 'World'])
    hello = Doc(vocab, words=['Hello'])

    matcher = Matcher(vocab)
    matcher.add('MyMatcher', None,
        [ {'ORTH': 'Hello' }, {'IS_ALPHA': True, 'OP': '?'} ])

    matches = matcher(hello_world)
    assert matches
    matches = matcher(hello)
    assert matches



# inspired from:
# https://python-packaging-user-guide.readthedocs.org/en/latest/single_source_version/
# https://github.com/pypa/warehouse/blob/master/warehouse/__about__.py

__title__ = 'spacy'
__version__ = '2.0.9'
__summary__ = 'Industrial-strength Natural Language Processing (NLP) with Python and Cython'
__uri__ = 'https://spacy.io'
__author__ = 'Explosion AI'
__email__ = 'contact@explosion.ai'
__license__ = 'MIT'
__release__ = True

__docs_models__ = 'https://spacy.io/usage/models'
__download_url__ = 'https://github.com/explosion/spacy-models/releases/download'
__compatibility__ = 'https://raw.githubusercontent.com/explosion/spacy-models/master/compatibility.json'
__shortcuts__ = 'https://raw.githubusercontent.com/explosion/spacy-models/master/shortcuts-v2.json'



# coding: utf8
from __future__ import unicode_literals

class IrishMorph:
    consonants = ['b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's', 't', 'v', 'w', 'x', 'z']
    broad_vowels = ['a', 'á', 'o', 'ó', 'u', 'ú']
    slender_vowels = ['e', 'é', 'i', 'í']
    vowels = broad_vowels + slender_vowels

    def ends_dentals(word):
        if word != "" and word[-1] in ['d', 'n', 't', 's']:
            return True
        else:
            return False

    def devoice(word):
        if len(word) > 2 and word[-2] == 's' and word[-1] == 'd':
            return word[:-1] + 't'
        else:
            return word

    def ends_with_vowel(word):
        return word != "" and word[-1] in vowels

    def starts_with_vowel(word):
        return word != "" and word[0] in vowels

    def deduplicate(word):
        if len(word) > 2 and word[-2] == word[-1] and word[-1] in consonants:
            return word[:-1]
        else:
            return word




'''Test Span.as_doc() doesn't segfault'''
from __future__ import unicode_literals
from ...tokens import Doc 
from ...vocab import Vocab
from ... import load as load_spacy


def test_issue1537():
    string = 'The sky is blue . The man is pink . The dog is purple .'
    doc = Doc(Vocab(), words=string.split())
    doc[0].sent_start = True
    for word in doc[1:]:
        if word.nbor(-1).text == '.':
            word.sent_start = True
        else:
            word.sent_start = False

    sents = list(doc.sents)
    sent0 = sents[0].as_doc()
    sent1 = sents[1].as_doc()
    assert isinstance(sent0, Doc)
    assert isinstance(sent1, Doc)


# Currently segfaulting, due to l_edge and r_edge misalignment
#def test_issue1537_model():
#    nlp = load_spacy('en')
#    doc = nlp(u'The sky is blue. The man is pink. The dog is purple.')
#    sents = [s.as_doc() for s in doc.sents]
#    print(list(sents[0].noun_chunks))
#    print(list(sents[1].noun_chunks))



# coding: utf8
from __future__ import unicode_literals

from ...compat import json_dumps, path2str
from ...util import prints
from ...gold import iob_to_biluo


def conll_ner2json(input_path, output_path, n_sents=10, use_morphology=False):
    """
    Convert files in the CoNLL-2003 NER format into JSON format for use with
    train cli.
    """
    docs = read_conll_ner(input_path)

    output_filename = input_path.parts[-1].replace(".conll", "") + ".json"
    output_filename = input_path.parts[-1].replace(".conll", "") + ".json"
    output_file = output_path / output_filename
    with output_file.open('w', encoding='utf-8') as f:
        f.write(json_dumps(docs))
    prints("Created %d documents" % len(docs),
           title="Generated output file %s" % path2str(output_file))


def read_conll_ner(input_path):
    text = input_path.open('r', encoding='utf-8').read()
    i = 0
    delimit_docs = '-DOCSTART- -X- O O'
    output_docs = []
    for doc in text.strip().split(delimit_docs):
        doc = doc.strip()
        if not doc:
            continue
        output_doc = []
        for sent in doc.split('\n\n'):
            sent = sent.strip()
            if not sent:
                continue
            lines = [line.strip() for line in sent.split('\n') if line.strip()]
            words, tags, chunks, iob_ents = zip(*[line.split() for line in lines])
            biluo_ents = iob_to_biluo(iob_ents)
            output_doc.append({'tokens': [
                {'orth': w, 'tag': tag, 'ner': ent} for (w, tag, ent) in
                zip(words, tags, biluo_ents)
            ]})
        output_docs.append({
            'id': len(output_docs),
            'paragraphs': [{'sentences': output_doc}]
        })
        output_doc = []
    return output_docs



# coding: utf8
from __future__ import unicode_literals

from ...symbols import ORTH, LEMMA, TAG, NORM, PRON_LEMMA


_exc = {}
_exclude = ["Ill", "ill", "Its", "its", "Hell", "hell", "Shell", "shell",
               "Shed", "shed", "were", "Were", "Well", "well", "Whore", "whore"]


# Pronouns

for pron in ["i"]:
    for orth in [pron, pron.title()]:
        _exc[orth + "'m"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'m", LEMMA: "be", NORM: "am", TAG: "VBP", "tenspect": 1, "number": 1}]

        _exc[orth + "m"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "m", LEMMA: "be", TAG: "VBP", "tenspect": 1, "number": 1 }]

        _exc[orth + "'ma"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'m", LEMMA: "be", NORM: "am"},
            {ORTH: "a", LEMMA: "going to", NORM: "gonna"}]

        _exc[orth + "ma"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "m", LEMMA: "be", NORM: "am"},
            {ORTH: "a", LEMMA: "going to", NORM: "gonna"}]


for pron in ["i", "you", "he", "she", "it", "we", "they"]:
    for orth in [pron, pron.title()]:
        _exc[orth + "'ll"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'ll", LEMMA: "will", NORM: "will", TAG: "MD"}]

        _exc[orth + "ll"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "ll", LEMMA: "will", NORM: "will", TAG: "MD"}]

        _exc[orth + "'ll've"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'ll", LEMMA: "will", NORM: "will", TAG: "MD"},
            {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "llve"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "ll", LEMMA: "will", NORM: "will", TAG: "MD"},
            {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "'d"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'d", LEMMA: "would", NORM: "would", TAG: "MD"}]

        _exc[orth + "d"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "d", LEMMA: "would", NORM: "would", TAG: "MD"}]

        _exc[orth + "'d've"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'d", LEMMA: "would", NORM: "would", TAG: "MD"},
            {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "dve"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "d", LEMMA: "would", NORM: "would", TAG: "MD"},
            {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}]


for pron in ["i", "you", "we", "they"]:
    for orth in [pron, pron.title()]:
        _exc[orth + "'ve"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "ve"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}]


for pron in ["you", "we", "they"]:
    for orth in [pron, pron.title()]:
        _exc[orth + "'re"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'re", LEMMA: "be", NORM: "are"}]

        _exc[orth + "re"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "re", LEMMA: "be", NORM: "are", TAG: "VBZ"}]


for pron in ["he", "she", "it"]:
    for orth in [pron, pron.title()]:
        _exc[orth + "'s"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "'s", NORM: "'s"}]

        _exc[orth + "s"] = [
            {ORTH: orth, LEMMA: PRON_LEMMA, NORM: pron, TAG: "PRP"},
            {ORTH: "s"}]


# W-words, relative pronouns, prepositions etc.

for word in ["who", "what", "when", "where", "why", "how", "there", "that"]:
    for orth in [word, word.title()]:
        _exc[orth + "'s"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "'s", NORM: "'s"}]

        _exc[orth + "s"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "s"}]

        _exc[orth + "'ll"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "'ll", LEMMA: "will", NORM: "will", TAG: "MD"}]

        _exc[orth + "ll"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "ll", LEMMA: "will", NORM: "will", TAG: "MD"}]

        _exc[orth + "'ll've"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "'ll", LEMMA: "will", NORM: "will", TAG: "MD"},
            {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "llve"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "ll", LEMMA: "will", NORM: "will", TAG: "MD"},
            {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "'re"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "'re", LEMMA: "be", NORM: "are"}]

        _exc[orth + "re"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "re", LEMMA: "be", NORM: "are"}]

        _exc[orth + "'ve"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "'ve", LEMMA: "have", TAG: "VB"}]

        _exc[orth + "ve"] = [
            {ORTH: orth, LEMMA: word},
            {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "'d"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "'d", NORM: "'d"}]

        _exc[orth + "d"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "d"}]

        _exc[orth + "'d've"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "'d", LEMMA: "would", NORM: "would", TAG: "MD"},
            {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[orth + "dve"] = [
            {ORTH: orth, LEMMA: word, NORM: word},
            {ORTH: "d", LEMMA: "would", NORM: "would", TAG: "MD"},
            {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}]


# Verbs

for verb_data in [
    {ORTH: "ca", LEMMA: "can", NORM: "can", TAG: "MD"},
    {ORTH: "could", NORM: "could", TAG: "MD"},
    {ORTH: "do", LEMMA: "do", NORM: "do"},
    {ORTH: "does", LEMMA: "do", NORM: "does"},
    {ORTH: "did", LEMMA: "do", NORM: "do", TAG: "VBD"},
    {ORTH: "had", LEMMA: "have", NORM: "have", TAG: "VBD"},
    {ORTH: "may", NORM: "may", TAG: "MD"},
    {ORTH: "might", NORM: "might", TAG: "MD"},
    {ORTH: "must", NORM: "must", TAG: "MD"},
    {ORTH: "need", NORM: "need"},
    {ORTH: "ought", NORM: "ought", TAG: "MD"},
    {ORTH: "sha", LEMMA: "shall", NORM: "shall", TAG: "MD"},
    {ORTH: "should", NORM: "should", TAG: "MD"},
    {ORTH: "wo", LEMMA: "will", NORM: "will", TAG: "MD"},
    {ORTH: "would", NORM: "would", TAG: "MD"}]:
    verb_data_tc = dict(verb_data)
    verb_data_tc[ORTH] = verb_data_tc[ORTH].title()
    for data in [verb_data, verb_data_tc]:
        _exc[data[ORTH] + "n't"] = [
            dict(data),
            {ORTH: "n't", LEMMA: "not", NORM: "not", TAG: "RB"}]

        _exc[data[ORTH] + "nt"] = [
            dict(data),
            {ORTH: "nt", LEMMA: "not", NORM: "not", TAG: "RB"}]

        _exc[data[ORTH] + "n't've"] = [
            dict(data),
            {ORTH: "n't", LEMMA: "not", NORM: "not", TAG: "RB"},
            {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}]

        _exc[data[ORTH] + "ntve"] = [
            dict(data),
            {ORTH: "nt", LEMMA: "not", NORM: "not", TAG: "RB"},
            {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}]


for verb_data in [
    {ORTH: "could", NORM: "could", TAG: "MD"},
    {ORTH: "might", NORM: "might", TAG: "MD"},
    {ORTH: "must", NORM: "must", TAG: "MD"},
    {ORTH: "should", NORM: "should", TAG: "MD"},
    {ORTH: "would", NORM: "would", TAG: "MD"}]:
    verb_data_tc = dict(verb_data)
    verb_data_tc[ORTH] = verb_data_tc[ORTH].title()
    for data in [verb_data, verb_data_tc]:
        _exc[data[ORTH] + "'ve"] = [
            dict(data),
            {ORTH: "'ve", LEMMA: "have", TAG: "VB"}]

        _exc[data[ORTH] + "ve"] = [
            dict(data),
            {ORTH: "ve", LEMMA: "have", TAG: "VB"}]


for verb_data in [
    {ORTH: "ai", LEMMA: "be", TAG: "VBP", "number": 2},
    {ORTH: "are", LEMMA: "be", NORM: "are", TAG: "VBP", "number": 2},
    {ORTH: "is", LEMMA: "be", NORM: "is", TAG: "VBZ"},
    {ORTH: "was", LEMMA: "be", NORM: "was"},
    {ORTH: "were", LEMMA: "be", NORM: "were"},
    {ORTH: "have", NORM: "have"},
    {ORTH: "has", LEMMA: "have", NORM: "has"},
    {ORTH: "dare", NORM: "dare"}]:
    verb_data_tc = dict(verb_data)
    verb_data_tc[ORTH] = verb_data_tc[ORTH].title()
    for data in [verb_data, verb_data_tc]:
        _exc[data[ORTH] + "n't"] = [
            dict(data),
            {ORTH: "n't", LEMMA: "not", NORM: "not", TAG: "RB"}]

        _exc[data[ORTH] + "nt"] = [
            dict(data),
            {ORTH: "nt", LEMMA: "not", NORM: "not", TAG: "RB"}]


# Other contractions with trailing apostrophe

for exc_data in [
    {ORTH: "doin", LEMMA: "do", NORM: "doing"},
    {ORTH: "goin", LEMMA: "go", NORM: "going"},
    {ORTH: "nothin", LEMMA: "nothing", NORM: "nothing"},
    {ORTH: "nuthin", LEMMA: "nothing", NORM: "nothing"},
    {ORTH: "ol", LEMMA: "old", NORM: "old"},
    {ORTH: "somethin", LEMMA: "something", NORM: "something"}]:
    exc_data_tc = dict(exc_data)
    exc_data_tc[ORTH] = exc_data_tc[ORTH].title()
    for data in [exc_data, exc_data_tc]:
        data_apos = dict(data)
        data_apos[ORTH] = data_apos[ORTH] + "'"
        _exc[data[ORTH]] = [dict(data)]
        _exc[data_apos[ORTH]] = [dict(data_apos)]


# Other contractions with leading apostrophe

for exc_data in [
    {ORTH: "cause", LEMMA: "because", NORM: "because"},
    {ORTH: "em", LEMMA: PRON_LEMMA, NORM: "them"},
    {ORTH: "ll", LEMMA: "will", NORM: "will"},
    {ORTH: "nuff", LEMMA: "enough", NORM: "enough"}]:
    exc_data_apos = dict(exc_data)
    exc_data_apos[ORTH] = "'" + exc_data_apos[ORTH]
    for data in [exc_data, exc_data_apos]:
        _exc[data[ORTH]] = [data]


# Times

for h in range(1, 12 + 1):
    for period in ["a.m.", "am"]:
        _exc["%d%s" % (h, period)] = [
            {ORTH: "%d" % h},
            {ORTH: period, LEMMA: "a.m.", NORM: "a.m."}]
    for period in ["p.m.", "pm"]:
        _exc["%d%s" % (h, period)] = [
            {ORTH: "%d" % h},
            {ORTH: period, LEMMA: "p.m.", NORM: "p.m."}]


# Rest

_other_exc = {
    "y'all": [
        {ORTH: "y'", LEMMA: PRON_LEMMA, NORM: "you"},
        {ORTH: "all"}],

    "yall": [
        {ORTH: "y", LEMMA: PRON_LEMMA, NORM: "you"},
        {ORTH: "all"}],

    "how'd'y": [
        {ORTH: "how", LEMMA: "how"},
        {ORTH: "'d", LEMMA: "do"},
        {ORTH: "'y", LEMMA: PRON_LEMMA, NORM: "you"}],

    "How'd'y": [
        {ORTH: "How", LEMMA: "how", NORM: "how"},
        {ORTH: "'d", LEMMA: "do"},
        {ORTH: "'y", LEMMA: PRON_LEMMA, NORM: "you"}],

    "not've": [
        {ORTH: "not", LEMMA: "not", TAG: "RB"},
        {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}],

    "notve": [
        {ORTH: "not", LEMMA: "not", TAG: "RB"},
        {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}],

    "Not've": [
        {ORTH: "Not", LEMMA: "not", NORM: "not", TAG: "RB"},
        {ORTH: "'ve", LEMMA: "have", NORM: "have", TAG: "VB"}],

    "Notve": [
        {ORTH: "Not", LEMMA: "not", NORM: "not", TAG: "RB"},
        {ORTH: "ve", LEMMA: "have", NORM: "have", TAG: "VB"}],

    "cannot": [
        {ORTH: "can", LEMMA: "can", TAG: "MD"},
        {ORTH: "not", LEMMA: "not", TAG: "RB"}],

    "Cannot": [
        {ORTH: "Can", LEMMA: "can", NORM: "can", TAG: "MD"},
        {ORTH: "not", LEMMA: "not", TAG: "RB"}],

    "gonna": [
        {ORTH: "gon", LEMMA: "go", NORM: "going"},
        {ORTH: "na", LEMMA: "to", NORM: "to"}],

    "Gonna": [
        {ORTH: "Gon", LEMMA: "go", NORM: "going"},
        {ORTH: "na", LEMMA: "to", NORM: "to"}],

    "gotta": [
        {ORTH: "got"},
        {ORTH: "ta", LEMMA: "to", NORM: "to"}],

    "Gotta": [
        {ORTH: "Got", NORM: "got"},
        {ORTH: "ta", LEMMA: "to", NORM: "to"}],

    "let's": [
        {ORTH: "let"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, NORM: "us"}],

    "Let's": [
        {ORTH: "Let", LEMMA: "let", NORM: "let"},
        {ORTH: "'s", LEMMA: PRON_LEMMA, NORM: "us"}]
}

_exc.update(_other_exc)


for exc_data in [
    {ORTH: "'S", LEMMA: "'s", NORM: "'s"},
    {ORTH: "'s", LEMMA: "'s", NORM: "'s"},
    {ORTH: "\u2018S", LEMMA: "'s", NORM: "'s"},
    {ORTH: "\u2018s", LEMMA: "'s", NORM: "'s"},
    {ORTH: "and/or", LEMMA: "and/or", NORM: "and/or", TAG: "CC"},
    {ORTH: "w/o", LEMMA: "without", NORM: "without"},
    {ORTH: "'re", LEMMA: "be", NORM: "are"},
    {ORTH: "'Cause", LEMMA: "because", NORM: "because"},
    {ORTH: "'cause", LEMMA: "because", NORM: "because"},
    {ORTH: "'cos", LEMMA: "because", NORM: "because"},
    {ORTH: "'Cos", LEMMA: "because", NORM: "because"},
    {ORTH: "'coz", LEMMA: "because", NORM: "because"},
    {ORTH: "'Coz", LEMMA: "because", NORM: "because"},
    {ORTH: "'cuz", LEMMA: "because", NORM: "because"},
    {ORTH: "'Cuz", LEMMA: "because", NORM: "because"},
    {ORTH: "'bout", LEMMA: "about", NORM: "about"},
    {ORTH: "ma'am", LEMMA: "madam", NORM: "madam"},
    {ORTH: "Ma'am", LEMMA: "madam", NORM: "madam"},
    {ORTH: "o'clock", LEMMA: "o'clock", NORM: "o'clock"},
    {ORTH: "O'clock", LEMMA: "o'clock", NORM: "o'clock"},
    {ORTH: "lovin'", LEMMA: "love", NORM: "loving"},
    {ORTH: "Lovin'", LEMMA: "love", NORM: "loving"},
    {ORTH: "lovin", LEMMA: "love", NORM: "loving"},
    {ORTH: "Lovin", LEMMA: "love", NORM: "loving"},
    {ORTH: "havin'", LEMMA: "have", NORM: "having"},
    {ORTH: "Havin'", LEMMA: "have", NORM: "having"},
    {ORTH: "havin", LEMMA: "have", NORM: "having"},
    {ORTH: "Havin", LEMMA: "have", NORM: "having"},
    {ORTH: "doin'", LEMMA: "do", NORM: "doing"},
    {ORTH: "Doin'", LEMMA: "do", NORM: "doing"},
    {ORTH: "doin", LEMMA: "do", NORM: "doing"},
    {ORTH: "Doin", LEMMA: "do", NORM: "doing"},
    {ORTH: "goin'", LEMMA: "go", NORM: "going"},
    {ORTH: "Goin'", LEMMA: "go", NORM: "going"},
    {ORTH: "goin", LEMMA: "go", NORM: "going"},
    {ORTH: "Goin", LEMMA: "go", NORM: "going"},


    {ORTH: "Mt.", LEMMA: "Mount", NORM: "Mount"},
    {ORTH: "Ak.", LEMMA: "Alaska", NORM: "Alaska"},
    {ORTH: "Ala.", LEMMA: "Alabama", NORM: "Alabama"},
    {ORTH: "Apr.", LEMMA: "April", NORM: "April"},
    {ORTH: "Ariz.", LEMMA: "Arizona", NORM: "Arizona"},
    {ORTH: "Ark.", LEMMA: "Arkansas", NORM: "Arkansas"},
    {ORTH: "Aug.", LEMMA: "August", NORM: "August"},
    {ORTH: "Calif.", LEMMA: "California", NORM: "California"},
    {ORTH: "Colo.", LEMMA: "Colorado", NORM: "Colorado"},
    {ORTH: "Conn.", LEMMA: "Connecticut", NORM: "Connecticut"},
    {ORTH: "Dec.", LEMMA: "December", NORM: "December"},
    {ORTH: "Del.", LEMMA: "Delaware", NORM: "Delaware"},
    {ORTH: "Feb.", LEMMA: "February", NORM: "February"},
    {ORTH: "Fla.", LEMMA: "Florida", NORM: "Florida"},
    {ORTH: "Ga.", LEMMA: "Georgia", NORM: "Georgia"},
    {ORTH: "Ia.", LEMMA: "Iowa", NORM: "Iowa"},
    {ORTH: "Id.", LEMMA: "Idaho", NORM: "Idaho"},
    {ORTH: "Ill.", LEMMA: "Illinois", NORM: "Illinois"},
    {ORTH: "Ind.", LEMMA: "Indiana", NORM: "Indiana"},
    {ORTH: "Jan.", LEMMA: "January", NORM: "January"},
    {ORTH: "Jul.", LEMMA: "July", NORM: "July"},
    {ORTH: "Jun.", LEMMA: "June", NORM: "June"},
    {ORTH: "Kan.", LEMMA: "Kansas", NORM: "Kansas"},
    {ORTH: "Kans.", LEMMA: "Kansas", NORM: "Kansas"},
    {ORTH: "Ky.", LEMMA: "Kentucky", NORM: "Kentucky"},
    {ORTH: "La.", LEMMA: "Louisiana", NORM: "Louisiana"},
    {ORTH: "Mar.", LEMMA: "March", NORM: "March"},
    {ORTH: "Mass.", LEMMA: "Massachusetts", NORM: "Massachusetts"},
    {ORTH: "May.", LEMMA: "May", NORM: "May"},
    {ORTH: "Mich.", LEMMA: "Michigan", NORM: "Michigan"},
    {ORTH: "Minn.", LEMMA: "Minnesota", NORM: "Minnesota"},
    {ORTH: "Miss.", LEMMA: "Mississippi", NORM: "Mississippi"},
    {ORTH: "N.C.", LEMMA: "North Carolina", NORM: "North Carolina"},
    {ORTH: "N.D.", LEMMA: "North Dakota", NORM: "North Dakota"},
    {ORTH: "N.H.", LEMMA: "New Hampshire", NORM: "New Hampshire"},
    {ORTH: "N.J.", LEMMA: "New Jersey", NORM: "New Jersey"},
    {ORTH: "N.M.", LEMMA: "New Mexico", NORM: "New Mexico"},
    {ORTH: "N.Y.", LEMMA: "New York", NORM: "New York"},
    {ORTH: "Neb.", LEMMA: "Nebraska", NORM: "Nebraska"},
    {ORTH: "Nebr.", LEMMA: "Nebraska", NORM: "Nebraska"},
    {ORTH: "Nev.", LEMMA: "Nevada", NORM: "Nevada"},
    {ORTH: "Nov.", LEMMA: "November", NORM: "November"},
    {ORTH: "Oct.", LEMMA: "October", NORM: "October"},
    {ORTH: "Okla.", LEMMA: "Oklahoma", NORM: "Oklahoma"},
    {ORTH: "Ore.", LEMMA: "Oregon", NORM: "Oregon"},
    {ORTH: "Pa.", LEMMA: "Pennsylvania", NORM: "Pennsylvania"},
    {ORTH: "S.C.", LEMMA: "South Carolina", NORM: "South Carolina"},
    {ORTH: "Sep.", LEMMA: "September", NORM: "September"},
    {ORTH: "Sept.", LEMMA: "September", NORM: "September"},
    {ORTH: "Tenn.", LEMMA: "Tennessee", NORM: "Tennessee"},
    {ORTH: "Va.", LEMMA: "Virginia", NORM: "Virginia"},
    {ORTH: "Wash.", LEMMA: "Washington", NORM: "Washington"},
    {ORTH: "Wis.", LEMMA: "Wisconsin", NORM: "Wisconsin"}]:
    _exc[exc_data[ORTH]] = [exc_data]


for orth in [
    "'d", "a.m.", "Adm.", "Bros.", "co.", "Co.", "Corp.", "D.C.", "Dr.", "e.g.",
    "E.g.", "E.G.", "Gen.", "Gov.", "i.e.", "I.e.", "I.E.", "Inc.", "Jr.",
    "Ltd.", "Md.", "Messrs.", "Mo.", "Mont.", "Mr.", "Mrs.", "Ms.", "p.m.",
    "Ph.D.", "Rep.", "Rev.", "Sen.", "St.", "vs."]:
    _exc[orth] = [{ORTH: orth}]


for string in _exclude:
    if string in _exc:
        _exc.pop(string)


TOKENIZER_EXCEPTIONS = _exc



# coding: utf8
from __future__ import unicode_literals


VERBS_IRREG = {
    "abetted": ("abet",),
    "abetting": ("abet",),
    "abhorred": ("abhor",),
    "abhorring": ("abhor",),
    "abode": ("abide",),
    "abought": ("aby",),
    "about-shipped": ("about-ship",),
    "about-shipping": ("about-ship",),
    "abutted": ("abut",),
    "abutting": ("abut",),
    "abye": ("aby",),
    "accompanied": ("accompany",),
    "acetified": ("acetify",),
    "acidified": ("acidify",),
    "acquitted": ("acquit",),
    "acquitting": ("acquit",),
    "ad-libbed": ("ad-lib",),
    "ad-libbing": ("ad-lib",),
    "addrest": ("address",),
    "admitted": ("admit",),
    "admitting": ("admit",),
    "aerified": ("aerify",),
    "air-dried": ("air-dry",),
    "airdropped": ("airdrop",),
    "airdropping": ("airdrop",),
    "alkalified": ("alkalify",),
    "allied": ("ally",),
    "allotted": ("allot",),
    "allotting": ("allot",),
    "allowed_for": ("allow_for",),
    "allowing_for": ("allow_for",),
    "allows_for": ("allow_for",),
    "am": ("be",),
    "ammonified": ("ammonify",),
    "amnestied": ("amnesty",),
    "amplified": ("amplify",),
    "anglified": ("anglify",),
    "annulled": ("annul",),
    "annulling": ("annul",),
    "appalled": ("appal", "appall",),
    "appalling": ("appal", "appall",),
    "applied": ("apply",),
    "arcked": ("arc",),
    "arcking": ("arc",),
    "are": ("be",),
    "argufied": ("argufy",),
    "arisen": ("arise",),
    "arose": ("arise",),
    "ate": ("eat",),
    "atrophied": ("atrophy",),
    "averred": ("aver",),
    "averring": ("aver",),
    "awoke": ("awake",),
    "awoken": ("awake",),
    "babied": ("baby",),
    "baby-sat": ("baby-sit",),
    "baby-sitting": ("baby-sit",),
    "back-pedalled": ("back-pedal",),
    "back-pedalling": ("back-pedal",),
    "backbit": ("backbite",),
    "backbitten": ("backbite",),
    "backslid": ("backslide",),
    "backslidden": ("backslide",),
    "bade": ("bid",),
    "bagged": ("bag",),
    "bagging": ("bag",),
    "ballyragged": ("ballyrag",),
    "ballyragging": ("ballyrag",),
    "bandied": ("bandy",),
    "banned": ("ban",),
    "banning": ("ban",),
    "barred": ("bar",),
    "barrelled": ("barrel",),
    "barrelling": ("barrel",),
    "barring": ("bar",),
    "basified": ("basify",),
    "batted": ("bat",),
    "batting": ("bat",),
    "bayonetted": ("bayonet",),
    "bayonetting": ("bayonet",),
    "beaten": ("beat",),
    "beatified": ("beatify",),
    "beautified": ("beautify",),
    "became": ("become",),
    "became_known": ("become_known",),
    "becomes_known": ("become_known",),
    "bed": ("bed",),
    "bedded": ("bed",),
    "bedding": ("bed",),
    "bedevilled": ("bedevil",),
    "bedevilling": ("bedevil",),
    "bedimmed": ("bedim",),
    "bedimming": ("bedim",),
    "been": ("be",),
    "befallen": ("befall",),
    "befell": ("befall",),
    "befitted": ("befit",),
    "befitting": ("befit",),
    "befogged": ("befog",),
    "befogging": ("befog",),
    "began": ("begin",),
    "begat": ("beget",),
    "begetting": ("beget",),
    "begged": ("beg",),
    "begging": ("beg",),
    "beginning": ("begin",),
    "begirt": ("begird",),
    "begot": ("beget",),
    "begotten": ("beget",),
    "begun": ("begin",),
    "beheld": ("behold",),
    "beholden": ("behold",),
    "bejewelled": ("bejewel",),
    "bejewelling": ("bejewel",),
    "bellied": ("belly",),
    "belly-flopped": ("belly-flop",),
    "belly-flopping": ("belly-flop",),
    "belying": ("belie",),
    "benefitted": ("benefit",),
    "benefitting": ("benefit",),
    "benempt": ("bename",),
    "bent": ("bend",),
    "berried": ("berry",),
    "besetting": ("beset",),
    "besought": ("beseech",),
    "bespoke": ("bespeak",),
    "bespoken": ("bespeak",),
    "bestirred": ("bestir",),
    "bestirring": ("bestir",),
    "bestrewn": ("bestrew",),
    "bestrid": ("bestride",),
    "bestridden": ("bestride",),
    "bestrode": ("bestride",),
    "betaken": ("betake",),
    "bethought": ("bethink",),
    "betook": ("betake",),
    "betted": ("bet",),
    "betting": ("bet",),
    "bevelled": ("bevel",),
    "bevelling": ("bevel",),
    "biassed": ("bias",),
    "biassing": ("bias",),
    "bidden": ("bid",),
    "bidding": ("bid",),
    "bing": ("bing",),
    "binned": ("bin",),
    "binning": ("bin",),
    "bird-dogged": ("bird-dog",),
    "bird-dogging": ("bird-dog",),
    "bit": ("bite",),
    "bitted": ("bit",),
    "bitten": ("bite",),
    "bitting": ("bit",),
    "bivouacked": ("bivouac",),
    "bivouacking": ("bivouac",),
    "blabbed": ("blab",),
    "blabbing": ("blab",),
    "blackberried": ("blackberry",),
    "blacklegged": ("blackleg",),
    "blacklegging": ("blackleg",),
    "blatted": ("blat",),
    "blatting": ("blat",),
    "bled": ("bleed",),
    "blest": ("bless",),
    "blew": ("blow",),
    "blew_one's_nose": ("blow_one's_nose",),
    "blipped": ("blip",),
    "blipping": ("blip",),
    "blobbed": ("blob",),
    "blobbing": ("blob",),
    "bloodied": ("bloody",),
    "blotted": ("blot",),
    "blotting": ("blot",),
    "blowing_one's_nose": ("blow_one's_nose",),
    "blown": ("blow",),
    "blows_one's_nose": ("blow_one's_nose",),
    "blubbed": ("blub",),
    "blubbing": ("blub",),
    "blue-pencilled": ("blue-pencil",),
    "blue-pencilling": ("blue-pencil",),
    "blurred": ("blur",),
    "blurring": ("blur",),
    "bobbed": ("bob",),
    "bobbing": ("bob",),
    "bodied": ("body",),
    "bogged-down": ("bog-down",),
    "bogged_down": ("bog_down",),
    "bogging-down": ("bog-down",),
    "bogging_down": ("bog_down",),
    "bogs-down": ("bog-down",),
    "bogs_down": ("bog_down",),
    "booby-trapped": ("booby-trap",),
    "booby-trapping": ("booby-trap",),
    "bootlegged": ("bootleg",),
    "bootlegging": ("bootleg",),
    "bopped": ("bop",),
    "bopping": ("bop",),
    "bore": ("bear",),
    "born": ("bear",),
    "borne": ("bear",),
    "bottle-fed": ("bottle-feed",),
    "bought": ("buy",),
    "bound": ("bind",),
    "bragged": ("brag",),
    "bragging": ("brag",),
    "breast-fed": ("breast-feed",),
    "bred": ("breed",),
    "brevetted": ("brevet",),
    "brevetting": ("brevet",),
    "brimmed": ("brim",),
    "brimming": ("brim",),
    "broke": ("break",),
    "broken": ("break",),
    "brought": ("bring",),
    "browbeaten": ("browbeat",),
    "brutified": ("brutify",),
    "budded": ("bud",),
    "budding": ("bud",),
    "bugged": ("bug",),
    "bugging": ("bug",),
    "built": ("build",),
    "bulldogging": ("bulldog",),
    "bullied": ("bully",),
    "bullshitted": ("bullshit",),
    "bullshitting": ("bullshit",),
    "bullwhipped": ("bullwhip",),
    "bullwhipping": ("bullwhip",),
    "bullyragged": ("bullyrag",),
    "bullyragging": ("bullyrag",),
    "bummed": ("bum",),
    "bumming": ("bum",),
    "buried": ("bury",),
    "burnt": ("burn",),
    "burred": ("bur",),
    "burring": ("bur",),
    "bushelled": ("bushel",),
    "bushelling": ("bushel",),
    "busied": ("busy",),
    "bypast": ("bypass",),
    "caballed": ("cabal",),
    "caballing": ("cabal",),
    "caddied": ("caddie", "caddy",),
    "caddies": ("caddie", "caddy",),
    "caddying": ("caddie", "caddy",),
    "calcified": ("calcify",),
    "came": ("come",),
    "canalled": ("canal",),
    "canalling": ("canal",),
    "cancelled": ("cancel",),
    "cancelling": ("cancel",),
    "candied": ("candy",),
    "canned": ("can",),
    "canning": ("can",),
    "canopied": ("canopy",),
    "capped": ("cap",),
    "capping": ("cap",),
    "carburetted": ("carburet",),
    "carburetting": ("carburet",),
    "carillonned": ("carillon",),
    "carillonning": ("carillon",),
    "carnied": ("carny",),
    "carnified": ("carnify",),
    "carolled": ("carol",),
    "carolling": ("carol",),
    "carried": ("carry",),
    "casefied": ("casefy",),
    "catnapped": ("catnap",),
    "catnapping": ("catnap",),
    "catted": ("cat",),
    "catting": ("cat",),
    "caught": ("catch",),
    "cavilled": ("cavil",),
    "cavilling": ("cavil",),
    "certified": ("certify",),
    "channelled": ("channel",),
    "channelling": ("channel",),
    "chapped": ("chap",),
    "chapping": ("chap",),
    "charred": ("char",),
    "charring": ("char",),
    "chatted": ("chat",),
    "chatting": ("chat",),
    "chevied": ("chivy",),
    "chevies": ("chivy",),
    "chevying": ("chivy",),
    "chid": ("chide",),
    "chidden": ("chide",),
    "chinned": ("chin",),
    "chinning": ("chin",),
    "chipped": ("chip",),
    "chipping": ("chip",),
    "chiselled": ("chisel",),
    "chiselling": ("chisel",),
    "chitchatted": ("chitchat",),
    "chitchatting": ("chitchat",),
    "chivied": ("chivy",),
    "chivved": ("chiv",),
    "chivvied": ("chivy",),
    "chivvies": ("chivy",),
    "chivving": ("chiv",),
    "chivvying": ("chivy",),
    "chondrified": ("chondrify",),
    "chopped": ("chop",),
    "chopping": ("chop",),
    "chose": ("choose",),
    "chosen": ("choose",),
    "chugged": ("chug",),
    "chugging": ("chug",),
    "chummed": ("chum",),
    "chumming": ("chum",),
    "citified": ("citify",),
    "clad": ("clothe",),
    "cladding": ("clad",),
    "clammed": ("clam",),
    "clamming": ("clam",),
    "clapped": ("clap",),
    "clapping": ("clap",),
    "clarified": ("clarify",),
    "classified": ("classify",),
    "cleft": ("cleave",),
    "clemmed": ("clem",),
    "clemming": ("clem",),
    "clept": ("clepe",),
    "clipped": ("clip",),
    "clipping": ("clip",),
    "clogged": ("clog",),
    "clogging": ("clog",),
    "clopped": ("clop",),
    "clopping": ("clop",),
    "clotted": ("clot",),
    "clotting": ("clot",),
    "clove": ("cleave",),
    "cloven": ("cleave",),
    "clubbed": ("club",),
    "clubbing": ("club",),
    "clung": ("cling",),
    "co-opted": ("coopt",),
    "co-opting": ("coopt",),
    "co-opts": ("coopts",),
    "co-ordinate": ("coordinate",),
    "co-ordinated": ("coordinate",),
    "co-ordinates": ("coordinate",),
    "co-ordinating": ("coordinate",),
    "co-starred": ("co-star",),
    "co-starring": ("co-star",),
    "cockneyfied": ("cockneyfy",),
    "codded": ("cod",),
    "codding": ("cod",),
    "codified": ("codify",),
    "cogged": ("cog",),
    "cogging": ("cog",),
    "coiffed": ("coif",),
    "coiffing": ("coif",),
    "collied": ("colly",),
    "combatted": ("combat",),
    "combatting": ("combat",),
    "committed": ("commit",),
    "committing": ("commit",),
    "compelled": ("compel",),
    "compelling": ("compel",),
    "complied": ("comply",),
    "complotted": ("complot",),
    "complotting": ("complot",),
    "concurred": ("concur",),
    "concurring": ("concur",),
    "confabbed": ("confab",),
    "confabbing": ("confab",),
    "conferred": ("confer",),
    "conferring": ("confer",),
    "conned": ("con",),
    "conning": ("con",),
    "controlled": ("control",),
    "controlling": ("control",),
    "copied": ("copy",),
    "coping": ("cope",),
    "copped": ("cop",),
    "copping": ("cop",),
    "coquetted": ("coquet",),
    "coquetting": ("coquet",),
    "corralled": ("corral",),
    "corralling": ("corral",),
    "counselled": ("counsel",),
    "counselling": ("counsel",),
    "counterplotted": ("counterplot",),
    "counterplotting": ("counterplot",),
    "countersank": ("countersink",),
    "countersunk": ("countersink",),
    "court-martialled": ("court-martial",),
    "court-martialling": ("court-martial",),
    "crabbed": ("crab",),
    "crabbing": ("crab",),
    "crammed": ("cram",),
    "cramming": ("cram",),
    "crapped": ("crap",),
    "crapping": ("crap",),
    "crept": ("creep",),
    "cribbed": ("crib",),
    "cribbing": ("crib",),
    "cried": ("cry",),
    "cropped": ("crop",),
    "cropping": ("crop",),
    "crossbred": ("crossbreed",),
    "crosscutting": ("crosscut",),
    "crucified": ("crucify",),
    "cubbed": ("cub",),
    "cubbing": ("cub",),
    "cudgelled": ("cudgel",),
    "cudgelling": ("cudgel",),
    "cupelled": ("cupel",),
    "cupelling": ("cupel",),
    "cupped": ("cup",),
    "cupping": ("cup",),
    "curetted": ("curet",),
    "curettes": ("curet",),
    "curetting": ("curet",),
    "curried": ("curry",),
    "curst": ("curse",),
    "curtsied": ("curtsy",),
    "curvetted": ("curvet",),
    "curvetting": ("curvet",),
    "cutting": ("cut",),
    "dabbed": ("dab",),
    "dabbing": ("dab",),
    "dagged": ("dag",),
    "dagging": ("dag",),
    "dallied": ("dally",),
    "dammed": ("dam",),
    "damming": ("dam",),
    "damnified": ("damnify",),
    "dandified": ("dandify",),
    "dapped": ("dap",),
    "dapping": ("dap",),
    "dealt": ("deal",),
    "debarred": ("debar",),
    "debarring": ("debar",),
    "debugged": ("debug",),
    "debugging": ("debug",),
    "debussed": ("debus",),
    "debusses": ("debus",),
    "debussing": ("debus",),
    "decalcified": ("decalcify",),
    "declassified": ("declassify",),
    "decontrolled": ("decontrol",),
    "decontrolling": ("decontrol",),
    "decried": ("decry",),
    "deep-freeze": ("deepfreeze",),
    "deep-freezed": ("deepfreeze",),
    "deep-freezes": ("deepfreeze",),
    "deep-fried": ("deep-fry",),
    "deferred": ("defer",),
    "deferring": ("defer",),
    "defied": ("defy",),
    "degassed": ("degas",),
    "degasses": ("degas",),
    "degassing": ("degas",),
    "dehumidified": ("dehumidify",),
    "deified": ("deify",),
    "demitted": ("demit",),
    "demitting": ("demit",),
    "demobbed": ("demob",),
    "demobbing": ("demob",),
    "demulsified": ("demulsify",),
    "demurred": ("demur",),
    "demurring": ("demur",),
    "demystified": ("demystify",),
    "denazified": ("denazify",),
    "denied": ("deny",),
    "denitrified": ("denitrify",),
    "denned": ("den",),
    "denning": ("den",),
    "descried": ("descry",),
    "deterred": ("deter",),
    "deterring": ("deter",),
    "detoxified": ("detoxify",),
    "devilled": ("devil",),
    "devilling": ("devil",),
    "devitrified": ("devitrify",),
    "diagrammed": ("diagram",),
    "diagramming": ("diagram",),
    "dialled": ("dial",),
    "dialling": ("dial",),
    "dibbed": ("dib",),
    "dibbing": ("dib",),
    "did": ("do",),
    "digging": ("dig",),
    "dignified": ("dignify",),
    "dilly-dallied": ("dilly-dally",),
    "dimmed": ("dim",),
    "dimming": ("dim",),
    "dinned": ("din",),
    "dinning": ("din",),
    "dipped": ("dip",),
    "dipping": ("dip",),
    "dirtied": ("dirty",),
    "disannulled": ("disannul",),
    "disannulling": ("disannul",),
    "disbarred": ("disbar",),
    "disbarring": ("disbar",),
    "disbudded": ("disbud",),
    "disbudding": ("disbud",),
    "disembodied": ("disembody",),
    "disembowelled": ("disembowel",),
    "disembowelling": ("disembowel",),
    "disenthralled": ("disenthral", "disenthrall",),
    "disenthralling": ("disenthral", "disenthrall",),
    "disenthralls": ("disenthral",),
    "disenthrals": ("disenthrall",),
    "dishevelled": ("dishevel",),
    "dishevelling": ("dishevel",),
    "disinterred": ("disinter",),
    "disinterring": ("disinter",),
    "dispelled": ("dispel",),
    "dispelling": ("dispel",),
    "disqualified": ("disqualify",),
    "dissatisfied": ("dissatisfy",),
    "distilled": ("distil", "distill",),
    "distilling": ("distil", "distill",),
    "diversified": ("diversify",),
    "divvied": ("divvy",),
    "dizzied": ("dizzy",),
    "dogged": ("dog",),
    "dogging": ("dog",),
    "doglegged": ("dogleg",),
    "doglegging": ("dogleg",),
    "dollied": ("dolly",),
    "done": ("do",),
    "donned": ("don",),
    "donning": ("don",),
    "dotted": ("dot",),
    "dotting": ("dot",),
    "dought": ("dow",),
    "dove": ("dive",),
    "drabbed": ("drab",),
    "drabbing": ("drab",),
    "dragged": ("drag",),
    "dragging": ("drag",),
    "drank": ("drink",),
    "drawn": ("draw",),
    "dreamt": ("dream",),
    "drew": ("draw",),
    "dried": ("dry",),
    "dripped": ("drip",),
    "dripping": ("drip",),
    "drivelled": ("drivel",),
    "drivelling": ("drivel",),
    "driven": ("drive",),
    "dropped": ("drop",),
    "dropping": ("drop",),
    "drove": ("drive",),
    "drubbed": ("drub",),
    "drubbing": ("drub",),
    "drugged": ("drug",),
    "drugging": ("drug",),
    "drummed": ("drum",),
    "drumming": ("drum",),
    "drunk": ("drink",),
    "dubbed": ("dub",),
    "dubbing": ("dub",),
    "duelled": ("duel",),
    "duelling": ("duel",),
    "dug": ("dig",),
    "dulcified": ("dulcify",),
    "dummied": ("dummy",),
    "dunned": ("dun",),
    "dunning": ("dun",),
    "dwelt": ("dwell",),
    "dying": ("die",),
    "easied": ("easy",),
    "eaten": ("eat",),
    "eavesdropped": ("eavesdrop",),
    "eavesdropping": ("eavesdrop",),
    "eddied": ("eddy",),
    "edified": ("edify",),
    "ego-tripped": ("ego-trip",),
    "ego-tripping": ("ego-trip",),
    "electrified": ("electrify",),
    "embedded": ("embed",),
    "embedding": ("embed",),
    "embodied": ("embody",),
    "embussed": ("embus",),
    "embusses": ("embus",),
    "embussing": ("embus",),
    "emitted": ("emit",),
    "emitting": ("emit",),
    "empanelled": ("empanel",),
    "empanelling": ("empanel",),
    "emptied": ("empty",),
    "emulsified": ("emulsify",),
    "enamelled": ("enamel",),
    "enamelling": ("enamel",),
    "englutted": ("englut",),
    "englutting": ("englut",),
    "enrolled": ("enrol", "enroll",),
    "enrolling": ("enrol", "enroll",),
    "enthralled": ("enthral", "enthrall",),
    "enthralling": ("enthral", "enthrall",),
    "entrammelled": ("entrammel",),
    "entrammelling": ("entrammel",),
    "entrapped": ("entrap",),
    "entrapping": ("entrap",),
    "envied": ("envy",),
    "enwound": ("enwind",),
    "enwrapped": ("enwrap",),
    "enwrapping": ("enwrap",),
    "equalled": ("equal",),
    "equalling": ("equal",),
    "equipped": ("equip",),
    "equipping": ("equip",),
    "espied": ("espy",),
    "esterified": ("esterify",),
    "estopped": ("estop",),
    "estopping": ("estop",),
    "etherified": ("etherify",),
    "excelled": ("excel",),
    "excelling": ("excel",),
    "exemplified": ("exemplify",),
    "expelled": ("expel",),
    "expelling": ("expel",),
    "extolled": ("extol", "extoll",),
    "extolling": ("extol", "extoll",),
    "facetted": ("facet",),
    "facetting": ("facet",),
    "fagged": ("fag",),
    "fagging": ("fag",),
    "fallen": ("fall",),
    "falsified": ("falsify",),
    "fancied": ("fancy",),
    "fanned": ("fan",),
    "fanning": ("fan",),
    "fantasied": ("fantasy",),
    "fatted": ("fat",),
    "fatting": ("fat",),
    "featherbedded": ("featherbed",),
    "featherbedding": ("featherbed",),
    "fed": ("feed",),
    "feed": ("feed", "fee",),
    "fell": ("fall",),
    "felt": ("feel",),
    "ferried": ("ferry",),
    "fibbed": ("fib",),
    "fibbing": ("fib",),
    "figged": ("fig",),
    "figging": ("fig",),
    "filled_up": ("fill_up",),
    "fine-drawn": ("fine-draw",),
    "fine-drew": ("fine-draw",),
    "finned": ("fin",),
    "finning": ("fin",),
    "fitted": ("fit",),
    "fitting": ("fit",),
    "flagged": ("flag",),
    "flagging": ("flag",),
    "flammed": ("flam",),
    "flamming": ("flam",),
    "flannelled": ("flannel",),
    "flannelling": ("flannel",),
    "flapped": ("flap",),
    "flapping": ("flap",),
    "flatted": ("flat",),
    "flatting": ("flat",),
    "fled": ("flee",),
    "flew": ("fly",),
    "flimflammed": ("flimflam",),
    "flimflamming": ("flimflam",),
    "flip-flopped": ("flip-flop",),
    "flip-flopping": ("flip-flop",),
    "flipped": ("flip",),
    "flipping": ("flip",),
    "flitted": ("flit",),
    "flitting": ("flit",),
    "flogged": ("flog",),
    "flogging": ("flog",),
    "floodlit": ("floodlight",),
    "flopped": ("flop",),
    "flopping": ("flop",),
    "flown": ("fly",),
    "flubbed": ("flub",),
    "flubbing": ("flub",),
    "flung": ("fling",),
    "flurried": ("flurry",),
    "flyblew": ("flyblow",),
    "flyblown": ("flyblow",),
    "fobbed": ("fob",),
    "fobbing": ("fob",),
    "fogged": ("fog",),
    "fogging": ("fog",),
    "footslogged": ("footslog",),
    "footslogging": ("footslog",),
    "forbad": ("forbid",),
    "forbade": ("forbid",),
    "forbidden": ("forbid",),
    "forbidding": ("forbid",),
    "forbore": ("forbear",),
    "forborne": ("forbear",),
    "force-fed": ("force-feed",),
    "fordid": ("fordo",),
    "fordone": ("fordo",),
    "foredid": ("foredo",),
    "foredone": ("foredo",),
    "foregone": ("forego",),
    "foreknew": ("foreknow",),
    "foreknown": ("foreknow",),
    "foreran": ("forerun",),
    "forerunning": ("forerun",),
    "foresaw": ("foresee",),
    "foreseen": ("foresee",),
    "foreshown": ("foreshow",),
    "forespoke": ("forespeak",),
    "forespoken": ("forespeak",),
    "foretold": ("foretell",),
    "forewent": ("forego",),
    "forgave": ("forgive",),
    "forgetting": ("forget",),
    "forgiven": ("forgive",),
    "forgone": ("forgo",),
    "forgot": ("forget",),
    "forgotten": ("forget",),
    "formatted": ("format",),
    "formatting": ("format",),
    "forsaken": ("forsake",),
    "forsook": ("forsake",),
    "forspoke": ("forspeak",),
    "forspoken": ("forspeak",),
    "forswore": ("forswear",),
    "forsworn": ("forswear",),
    "fortified": ("fortify",),
    "forwent": ("forgo",),
    "fought": ("fight",),
    "found": ("find",),
    "foxtrotted": ("foxtrot",),
    "foxtrotting": ("foxtrot",),
    "frapped": ("frap",),
    "frapping": ("frap",),
    "freeze-dried": ("freeze-dry",),
    "frenchified": ("frenchify",),
    "frenzied": ("frenzy",),
    "fretted": ("fret",),
    "fretting": ("fret",),
    "fried": ("fry",),
    "frigged": ("frig",),
    "frigging": ("frig",),
    "fritted": ("frit", "fritt",),
    "fritting": ("frit", "fritt",),
    "frivolled": ("frivol",),
    "frivolling": ("frivol",),
    "frogged": ("frog",),
    "frogging": ("frog",),
    "frolicked": ("frolic",),
    "frolicking": ("frolic",),
    "froze": ("freeze",),
    "frozen": ("freeze",),
    "fructified": ("fructify",),
    "fuelled": ("fuel",),
    "fuelling": ("fuel",),
    "fulfilled": ("fulfil", "fulfill",),
    "fulfilling": ("fulfil", "fulfill",),
    "funned": ("fun",),
    "funnelled": ("funnel",),
    "funnelling": ("funnel",),
    "funning": ("fun",),
    "furred": ("fur",),
    "furring": ("fur",),
    "gadded": ("gad",),
    "gadding": ("gad",),
    "gagged": ("gag",),
    "gagging": ("gag",),
    "gainsaid": ("gainsay",),
    "gambolled": ("gambol",),
    "gambolling": ("gambol",),
    "gammed": ("gam",),
    "gamming": ("gam",),
    "gan": ("gin",),
    "ganned": ("gan",),
    "ganning": ("gan",),
    "gapped": ("gap",),
    "gapping": ("gap",),
    "gasified": ("gasify",),
    "gassed": ("gas",),
    "gasses": ("gas",),
    "gassing": ("gas",),
    "gave": ("give",),
    "gelled": ("gel",),
    "gelling": ("gel",),
    "gelt": ("geld",),
    "gemmed": ("gem",),
    "gemming": ("gem",),
    "genned-up": ("gen-up",),
    "genning-up": ("gen-up",),
    "gens-up": ("gen-up",),
    "gets_lost": ("get_lost",),
    "gets_started": ("get_started",),
    "getting": ("get",),
    "getting_lost": ("get_lost",),
    "getting_started": ("get_started",),
    "ghostwritten": ("ghostwrite",),
    "ghostwrote": ("ghostwrite",),
    "gibbed": ("gib",),
    "gibbing": ("gib",),
    "giddied": ("giddy",),
    "giftwrapped": ("giftwrap",),
    "giftwrapping": ("giftwrap",),
    "gigged": ("gig",),
    "gigging": ("gig",),
    "gilt": ("gild",),
    "ginned": ("gin",),
    "ginning": ("gin",),
    "gipped": ("gip",),
    "gipping": ("gip",),
    "girt": ("gird",),
    "given": ("give",),
    "glommed": ("glom",),
    "glomming": ("glom",),
    "gloried": ("glory",),
    "glorified": ("glorify",),
    "glutted": ("glut",),
    "glutting": ("glut",),
    "gnawn": ("gnaw",),
    "goes_deep": ("go_deep",),
    "going_deep": ("go_deep",),
    "gollied": ("golly",),
    "gone": ("go",),
    "gone_deep": ("go_deep",),
    "goose-stepped": ("goose-step",),
    "goose-stepping": ("goose-step",),
    "got": ("get",),
    "got_lost": ("get_lost",),
    "got_started": ("get_started",),
    "gotten": ("get",),
    "gotten_lost": ("get_lost",),
    "grabbed": ("grab",),
    "grabbing": ("grab",),
    "gratified": ("gratify",),
    "gravelled": ("gravel",),
    "gravelling": ("gravel",),
    "graven": ("grave",),
    "grew": ("grow",),
    "grinned": ("grin",),
    "grinning": ("grin",),
    "gripped": ("grip",),
    "gripping": ("grip",),
    "gript": ("grip",),
    "gritted": ("grit",),
    "gritting": ("grit",),
    "ground": ("grind",),
    "grovelled": ("grovel",),
    "grovelling": ("grovel",),
    "grown": ("grow",),
    "grubbed": ("grub",),
    "grubbing": ("grub",),
    "guarantied": ("guaranty",),
    "gullied": ("gully",),
    "gummed": ("gum",),
    "gumming": ("gum",),
    "gunned": ("gun",),
    "gunning": ("gun",),
    "gypped": ("gyp",),
    "gypping": ("gyp",),
    "hacksawn": ("hacksaw",),
    "had": ("have",),
    "had_a_feeling": ("have_a_feeling",),
    "had_left": ("have_left",),
    "had_the_feeling": ("have_the_feeling",),
    "hammed": ("ham",),
    "hamming": ("ham",),
    "hamstrung": ("hamstring",),
    "hand-knitted": ("hand-knit",),
    "hand-knitting": ("hand-knit",),
    "handfed": ("handfeed",),
    "handicapped": ("handicap",),
    "handicapping": ("handicap",),
    "handselled": ("handsel",),
    "handselling": ("handsel",),
    "harried": ("harry",),
    "has": ("have",),
    "has_a_feeling": ("have_a_feeling",),
    "has_left": ("have_left",),
    "has_the_feeling": ("have_the_feeling",),
    "hatchelled": ("hatchel",),
    "hatchelling": ("hatchel",),
    "hatted": ("hat",),
    "hatting": ("hat",),
    "having_a_feeling": ("have_a_feeling",),
    "having_left": ("have_left",),
    "having_the_feeling": ("have_the_feeling",),
    "heard": ("hear",),
    "hedgehopped": ("hedgehop",),
    "hedgehopping": ("hedgehop",),
    "held": ("hold",),
    "hemmed": ("hem",),
    "hemming": ("hem",),
    "hewn": ("hew",),
    "hiccupped": ("hiccup",),
    "hiccupping": ("hiccup",),
    "hid": ("hide",),
    "hidden": ("hide",),
    "high-hatted": ("high-hat",),
    "high-hatting": ("high-hat",),
    "hinnied": ("hinny",),
    "hitting": ("hit",),
    "hobbed": ("hob",),
    "hobbing": ("hob",),
    "hobnobbed": ("hobnob",),
    "hobnobbing": ("hobnob",),
    "hocus-pocussed": ("hocus-pocus",),
    "hocus-pocussing": ("hocus-pocus",),
    "hocussed": ("hocus",),
    "hocussing": ("hocus",),
    "hogged": ("hog",),
    "hogging": ("hog",),
    "hogtying": ("hogtie",),
    "honied": ("honey",),
    "hopped": ("hop",),
    "hopping": ("hop",),
    "horrified": ("horrify",),
    "horsewhipped": ("horsewhip",),
    "horsewhipping": ("horsewhip",),
    "houselled": ("housel",),
    "houselling": ("housel",),
    "hove": ("heave",),
    "hovelled": ("hovel",),
    "hovelling": ("hovel",),
    "hugged": ("hug",),
    "hugging": ("hug",),
    "humbugged": ("humbug",),
    "humbugging": ("humbug",),
    "humidified": ("humidify",),
    "hummed": ("hum",),
    "humming": ("hum",),
    "hung": ("hang",),
    "hurried": ("hurry",),
    "hypertrophied": ("hypertrophy",),
    "identified": ("identify",),
    "imbedded": ("imbed",),
    "imbedding": ("imbed",),
    "impanelled": ("impanel",),
    "impanelling": ("impanel",),
    "impelled": ("impel",),
    "impelling": ("impel",),
    "implied": ("imply",),
    "inbred": ("inbreed",),
    "incurred": ("incur",),
    "incurring": ("incur",),
    "indemnified": ("indemnify",),
    "indwelt": ("indwell",),
    "inferred": ("infer",),
    "inferring": ("infer",),
    "initialled": ("initial",),
    "initialling": ("initial",),
    "inlaid": ("inlay",),
    "insetting": ("inset",),
    "inspanned": ("inspan",),
    "inspanning": ("inspan",),
    "installed": ("instal", "install",),
    "installing": ("instal", "install",),
    "intensified": ("intensify",),
    "interbred": ("interbreed",),
    "intercropped": ("intercrop",),
    "intercropping": ("intercrop",),
    "intercutting": ("intercut",),
    "interlaid": ("interlay",),
    "interlapped": ("interlap",),
    "interlapping": ("interlap",),
    "intermarried": ("intermarry",),
    "intermitted": ("intermit",),
    "intermitting": ("intermit",),
    "interpled": ("interplead",),
    "interred": ("inter",),
    "interring": ("inter",),
    "interstratified": ("interstratify",),
    "interwove": ("interweave",),
    "interwoven": ("interweave",),
    "intromitted": ("intromit",),
    "intromitting": ("intromit",),
    "inwove": ("inweave",),
    "inwoven": ("inweave",),
    "inwrapped": ("inwrap",),
    "inwrapping": ("inwrap",),
    "is": ("be",),
    "jabbed": ("jab",),
    "jabbing": ("jab",),
    "jagged": ("jag",),
    "jagging": ("jag",),
    "jammed": ("jam",),
    "jamming": ("jam",),
    "japanned": ("japan",),
    "japanning": ("japan",),
    "jarred": ("jar",),
    "jarring": ("jar",),
    "jellied": ("jelly",),
    "jellified": ("jellify",),
    "jemmied": ("jemmy",),
    "jerry-built": ("jerry-build",),
    "jetted": ("jet",),
    "jetting": ("jet",),
    "jewelled": ("jewel",),
    "jewelling": ("jewel",),
    "jibbed": ("jib",),
    "jibbing": ("jib",),
    "jigged": ("jig",),
    "jigging": ("jig",),
    "jimmied": ("jimmy",),
    "jitterbugged": ("jitterbug",),
    "jitterbugging": ("jitterbug",),
    "jobbed": ("job",),
    "jobbing": ("job",),
    "jog-trotted": ("jog-trot",),
    "jog-trotting": ("jog-trot",),
    "jogged": ("jog",),
    "jogging": ("jog",),
    "joined_battle": ("join_battle",),
    "joined_forces": ("join_forces",),
    "joining_battle": ("join_battle",),
    "joining_forces": ("join_forces",),
    "joins_battle": ("join_battle",),
    "joins_forces": ("join_forces",),
    "jollied": ("jolly",),
    "jollified": ("jollify",),
    "jotted": ("jot",),
    "jotting": ("jot",),
    "joy-ridden": ("joy-ride",),
    "joy-rode": ("joy-ride",),
    "joypopped": ("joypop",),
    "joypopping": ("joypop",),
    "jugged": ("jug",),
    "jugging": ("jug",),
    "jumped_off": ("jump_off",),
    "jumping_off": ("jump_off",),
    "jumps_off": ("jump_off",),
    "justified": ("justify",),
    "jutted": ("jut",),
    "jutting": ("jut",),
    "kenned": ("ken",),
    "kennelled": ("kennel",),
    "kennelling": ("kennel",),
    "kenning": ("ken",),
    "kent": ("ken",),
    "kept": ("keep",),
    "kernelled": ("kernel",),
    "kernelling": ("kernel",),
    "kidded": ("kid",),
    "kidding": ("kid",),
    "kidnapped": ("kidnap",),
    "kidnapping": ("kidnap",),
    "kipped": ("kip",),
    "kipping": ("kip",),
    "knapped": ("knap",),
    "knapping": ("knap",),
    "kneecapped": ("kneecap",),
    "kneecapping": ("kneecap",),
    "knelt": ("kneel",),
    "knew": ("know",),
    "knitted": ("knit",),
    "knitting": ("knit",),
    "knobbed": ("knob",),
    "knobbing": ("knob",),
    "knotted": ("knot",),
    "knotting": ("knot",),
    "known": ("know",),
    "ko'd": ("ko",),
    "ko'ing": ("ko",),
    "ko's": ("ko",),
    "labelled": ("label",),
    "labelling": ("label",),
    "laden": ("lade",),
    "ladyfied": ("ladify",),
    "ladyfies": ("ladify",),
    "ladyfying": ("ladify",),
    "lagged": ("lag",),
    "lagging": ("lag",),
    "laid": ("lay",),
    "lain": ("lie",),
    "lallygagged": ("lallygag",),
    "lallygagging": ("lallygag",),
    "lammed": ("lam",),
    "lamming": ("lam",),
    "lapidified": ("lapidify",),
    "lapped": ("lap",),
    "lapping": ("lap",),
    "laurelled": ("laurel",),
    "laurelling": ("laurel",),
    "lay": ("lie",),
    "layed_for": ("lie_for",),
    "laying_for": ("lie_for",),
    "lays_for": ("lie_for",),
    "leant": ("lean",),
    "leapfrogged": ("leapfrog",),
    "leapfrogging": ("leapfrog",),
    "leapt": ("leap",),
    "learnt": ("learn",),
    "leaves_undone": ("leave_undone",),
    "leaving_undone": ("leave_undone",),
    "led": ("lead",),
    "left": ("leave",),
    "left_undone": ("leave_undone",),
    "lent": ("lend",),
    "letting": ("let",),
    "levelled": ("level",),
    "levelling": ("level",),
    "levied": ("levy",),
    "libelled": ("libel",),
    "libelling": ("libel",),
    "lignified": ("lignify",),
    "lipped": ("lip",),
    "lipping": ("lip",),
    "liquefied": ("liquefy",),
    "liquified": ("liquify",),
    "lit": ("light",),
    "lobbed": ("lob",),
    "lobbied": ("lobby",),
    "lobbing": ("lob",),
    "logged": ("log",),
    "logging": ("log",),
    "looked_towards": ("look_towards",),
    "looking_towards": ("look_towards",),
    "looks_towards": ("look_towards",),
    "lopped": ("lop",),
    "lopping": ("lop",),
    "lost": ("lose",),
    "lotted": ("lot",),
    "lotting": ("lot",),
    "lugged": ("lug",),
    "lugging": ("lug",),
    "lullabied": ("lullaby",),
    "lying": ("lie",),
    "machine-gunned": ("machine-gun",),
    "machine-gunning": ("machine-gun",),
    "madded": ("mad",),
    "madding": ("mad",),
    "made": ("make",),
    "magnified": ("magnify",),
    "manned": ("man",),
    "manning": ("man",),
    "manumitted": ("manumit",),
    "manumitting": ("manumit",),
    "mapped": ("map",),
    "mapping": ("map",),
    "marcelled": ("marcel",),
    "marcelling": ("marcel",),
    "marred": ("mar",),
    "married": ("marry",),
    "marring": ("mar",),
    "marshalled": ("marshal",),
    "marshalling": ("marshal",),
    "marvelled": ("marvel",),
    "marvelling": ("marvel",),
    "matted": ("mat",),
    "matting": ("mat",),
    "meant": ("mean",),
    "medalled": ("medal",),
    "medalling": ("medal",),
    "met": ("meet",),
    "metalled": ("metal",),
    "metalling": ("metal",),
    "metrified": ("metrify",),
    "might": ("may",),
    "militated_against": ("militate_against",),
    "militates_against": ("militate_against",),
    "militating_against": ("militate_against",),
    "mimicked": ("mimic",),
    "mimicking": ("mimic",),
    "minified": ("minify",),
    "misapplied": ("misapply",),
    "misbecame": ("misbecome",),
    "miscarried": ("miscarry",),
    "misdealt": ("misdeal",),
    "misfitted": ("misfit",),
    "misfitting": ("misfit",),
    "misgave": ("misgive",),
    "misgiven": ("misgive",),
    "mishitting": ("mishit",),
    "mislaid": ("mislay",),
    "misled": ("mislead",),
    "mispled": ("misplead",),
    "misspelt": ("misspell",),
    "misspent": ("misspend",),
    "mistaken": ("mistake",),
    "mistook": ("mistake",),
    "misunderstood": ("misunderstand",),
    "mobbed": ("mob",),
    "mobbing": ("mob",),
    "modelled": ("model",),
    "modelling": ("model",),
    "modified": ("modify",),
    "mollified": ("mollify",),
    "molten": ("melt",),
    "mopped": ("mop",),
    "mopping": ("mop",),
    "mortified": ("mortify",),
    "mown": ("mow",),
    "mudded": ("mud",),
    "muddied": ("muddy",),
    "mudding": ("mud",),
    "mugged": ("mug",),
    "mugging": ("mug",),
    "multiplied": ("multiply",),
    "mummed": ("mum",),
    "mummified": ("mummify",),
    "mumming": ("mum",),
    "mutinied": ("mutiny",),
    "mystified": ("mystify",),
    "nabbed": ("nab",),
    "nabbing": ("nab",),
    "nagged": ("nag",),
    "nagging": ("nag",),
    "napped": ("nap",),
    "napping": ("nap",),
    "netted": ("net",),
    "netting": ("net",),
    "nibbed": ("nib",),
    "nibbing": ("nib",),
    "nickelled": ("nickel",),
    "nickelling": ("nickel",),
    "nid-nodded": ("nid-nod",),
    "nid-nodding": ("nid-nod",),
    "nidified": ("nidify",),
    "nigrified": ("nigrify",),
    "nipped": ("nip",),
    "nipping": ("nip",),
    "nitrified": ("nitrify",),
    "nodded": ("nod",),
    "nodding": ("nod",),
    "non-prossed": ("non-pros",),
    "non-prosses": ("non-pros",),
    "non-prossing": ("non-pros",),
    "nonplussed": ("nonplus",),
    "nonplusses": ("nonplus",),
    "nonplussing": ("nonplus",),
    "notified": ("notify",),
    "nullified": ("nullify",),
    "nutted": ("nut",),
    "nutting": ("nut",),
    "objectified": ("objectify",),
    "occupied": ("occupy",),
    "occurred": ("occur",),
    "occurring": ("occur",),
    "offsetting": ("offset",),
    "omitted": ("omit",),
    "omitting": ("omit",),
    "ossified": ("ossify",),
    "outbidden": ("outbid",),
    "outbidding": ("outbid",),
    "outbred": ("outbreed",),
    "outcried": ("outcry",),
    "outcropped": ("outcrop",),
    "outcropping": ("outcrop",),
    "outdid": ("outdo",),
    "outdone": ("outdo",),
    "outdrawn": ("outdraw",),
    "outdrew": ("outdraw",),
    "outfitted": ("outfit",),
    "outfitting": ("outfit",),
    "outfought": ("outfight",),
    "outgassed": ("outgas",),
    "outgasses": ("outgas",),
    "outgassing": ("outgas",),
    "outgeneralled": ("outgeneral",),
    "outgeneralling": ("outgeneral",),
    "outgone": ("outgo",),
    "outgrew": ("outgrow",),
    "outgrown": ("outgrow",),
    "outlaid": ("outlay",),
    "outmanned": ("outman",),
    "outmanning": ("outman",),
    "outputted": ("output",),
    "outputting": ("output",),
    "outran": ("outrun",),
    "outridden": ("outride",),
    "outrode": ("outride",),
    "outrunning": ("outrun",),
    "outshone": ("outshine",),
    "outshot": ("outshoot",),
    "outsold": ("outsell",),
    "outspanned": ("outspan",),
    "outspanning": ("outspan",),
    "outstood": ("outstand",),
    "outstripped": ("outstrip",),
    "outstripping": ("outstrip",),
    "outthought": ("outthink",),
    "outwent": ("outgo",),
    "outwitted": ("outwit",),
    "outwitting": ("outwit",),
    "outwore": ("outwear",),
    "outworn": ("outwear",),
    "overbidden": ("overbid",),
    "overbidding": ("overbid",),
    "overblew": ("overblow",),
    "overblown": ("overblow",),
    "overbore": ("overbear",),
    "overborne": ("overbear",),
    "overbuilt": ("overbuild",),
    "overcame": ("overcome",),
    "overcropped": ("overcrop",),
    "overcropping": ("overcrop",),
    "overdid": ("overdo",),
    "overdone": ("overdo",),
    "overdrawn": ("overdraw",),
    "overdrew": ("overdraw",),
    "overdriven": ("overdrive",),
    "overdrove": ("overdrive",),
    "overflew": ("overfly",),
    "overflown": ("overflow", "overfly",),
    "overgrew": ("overgrow",),
    "overgrown": ("overgrow",),
    "overheard": ("overhear",),
    "overhung": ("overhang",),
    "overlaid": ("overlay",),
    "overlain": ("overlie",),
    "overlapped": ("overlap",),
    "overlapping": ("overlap",),
    "overlay": ("overlie",),
    "overlying": ("overlie",),
    "overmanned": ("overman",),
    "overmanning": ("overman",),
    "overpaid": ("overpay",),
    "overpast": ("overpass",),
    "overran": ("overrun",),
    "overridden": ("override",),
    "overrode": ("override",),
    "overrunning": ("overrun",),
    "oversaw": ("oversee",),
    "overseen": ("oversee",),
    "oversetting": ("overset",),
    "oversewn": ("oversew",),
    "overshot": ("overshoot",),
    "oversimplified": ("oversimplify",),
    "overslept": ("oversleep",),
    "oversold": ("oversell",),
    "overspent": ("overspend",),
    "overspilt": ("overspill",),
    "overstepped": ("overstep",),
    "overstepping": ("overstep",),
    "overtaken": ("overtake",),
    "overthrew": ("overthrow",),
    "overthrown": ("overthrow",),
    "overtook": ("overtake",),
    "overtopped": ("overtop",),
    "overtopping": ("overtop",),
    "overwound": ("overwind",),
    "overwritten": ("overwrite",),
    "overwrote": ("overwrite",),
    "pacified": ("pacify",),
    "padded": ("pad",),
    "padding": ("pad",),
    "paid": ("pay",),
    "palled": ("pal",),
    "palling": ("pal",),
    "palsied": ("palsy",),
    "pandied": ("pandy",),
    "panelled": ("panel",),
    "panelling": ("panel",),
    "panicked": ("panic",),
    "panicking": ("panic",),
    "panned": ("pan",),
    "panning": ("pan",),
    "parallelled": ("parallel",),
    "parallelling": ("parallel",),
    "parcelled": ("parcel",),
    "parcelling": ("parcel",),
    "parodied": ("parody",),
    "parried": ("parry",),
    "partaken": ("partake",),
    "partook": ("partake",),
    "pasquil": ("pasquinade",),
    "pasquilled": ("pasquinade",),
    "pasquilling": ("pasquinade",),
    "pasquils": ("pasquinade",),
    "patrolled": ("patrol",),
    "patrolling": ("patrol",),
    "patted": ("pat",),
    "patting": ("pat",),
    "pedalled": ("pedal",),
    "pedalling": ("pedal",),
    "pegged": ("peg",),
    "pegging": ("peg",),
    "pencilled": ("pencil",),
    "pencilling": ("pencil",),
    "penned": ("pen",),
    "penning": ("pen",),
    "pent": ("pen",),
    "pepped": ("pep",),
    "pepping": ("pep",),
    "permitted": ("permit",),
    "permitting": ("permit",),
    "personified": ("personify",),
    "petrified": ("petrify",),
    "petted": ("pet",),
    "pettifogged": ("pettifog",),
    "pettifogging": ("pettifog",),
    "petting": ("pet",),
    "phantasied": ("phantasy",),
    "photocopied": ("photocopy",),
    "photomapped": ("photomap",),
    "photomapping": ("photomap",),
    "photosetting": ("photoset",),
    "physicked": ("physic",),
    "physicking": ("physic",),
    "picnicked": ("picnic",),
    "picnicking": ("picnic",),
    "pigged": ("pig",),
    "pigging": ("pig",),
    "pilloried": ("pillory",),
    "pinch-hitting": ("pinch-hit",),
    "pinned": ("pin",),
    "pinning": ("pin",),
    "pipped": ("pip",),
    "pipping": ("pip",),
    "pistol-whipped": ("pistol-whip",),
    "pistol-whipping": ("pistol-whip",),
    "pistolled": ("pistol",),
    "pistolling": ("pistol",),
    "pitapatted": ("pitapat",),
    "pitapatting": ("pitapat",),
    "pitied": ("pity",),
    "pitted": ("pit",),
    "pitting": ("pit",),
    "planned": ("plan",),
    "planning": ("plan",),
    "platted": ("plat",),
    "platting": ("plat",),
    "played_a_part": ("play_a_part",),
    "playing_a_part": ("play_a_part",),
    "plays_a_part": ("play_a_part",),
    "pled": ("plead",),
    "plied": ("ply",),
    "plodded": ("plod",),
    "plodding": ("plod",),
    "plopped": ("plop",),
    "plopping": ("plop",),
    "plotted": ("plot",),
    "plotting": ("plot",),
    "plugged": ("plug",),
    "plugging": ("plug",),
    "podded": ("pod",),
    "podding": ("pod",),
    "pommelled": ("pommel",),
    "pommelling": ("pommel",),
    "popes": ("popes",),
    "popped": ("pop",),
    "popping": ("pop",),
    "potted": ("pot",),
    "potting": ("pot",),
    "preachified": ("preachify",),
    "precancelled": ("precancel",),
    "precancelling": ("precancel",),
    "preferred": ("prefer",),
    "preferring": ("prefer",),
    "preoccupied": ("preoccupy",),
    "prepaid": ("prepay",),
    "presignified": ("presignify",),
    "pretermitted": ("pretermit",),
    "pretermitting": ("pretermit",),
    "prettied": ("pretty",),
    "prettified": ("prettify",),
    "pried": ("pry",),
    "prigged": ("prig",),
    "prigging": ("prig",),
    "primmed": ("prim",),
    "primming": ("prim",),
    "prodded": ("prod",),
    "prodding": ("prod",),
    "programmed": ("program",),
    "programmes": ("program",),
    "programming": ("program",),
    "prologed": ("prologue",),
    "prologing": ("prologue",),
    "prologs": ("prologue",),
    "propelled": ("propel",),
    "propelling": ("propel",),
    "prophesied": ("prophesy",),
    "propped": ("prop",),
    "propping": ("prop",),
    "proven": ("prove",),
    "pubbed": ("pub",),
    "pubbing": ("pub",),
    "pugged": ("pug",),
    "pugging": ("pug",),
    "pummelled": ("pummel",),
    "pummelling": ("pummel",),
    "punned": ("pun",),
    "punning": ("pun",),
    "pupped": ("pup",),
    "pupping": ("pup",),
    "purified": ("purify",),
    "put-putted": ("put-put",),
    "put-putting": ("put-put",),
    "putrefied": ("putrefy",),
    "puttied": ("putty",),
    "putting": ("put",),
    "qualified": ("qualify",),
    "quantified": ("quantify",),
    "quarrelled": ("quarrel",),
    "quarrelling": ("quarrel",),
    "quarried": ("quarry",),
    "quartersawn": ("quartersaw",),
    "queried": ("query",),
    "quick-froze": ("quick-freeze",),
    "quick-frozen": ("quick-freeze",),
    "quickstepped": ("quickstep",),
    "quickstepping": ("quickstep",),
    "quipped": ("quip",),
    "quipping": ("quip",),
    "quitted": ("quit",),
    "quitting": ("quit",),
    "quizzed": ("quiz",),
    "quizzes": ("quiz",),
    "quizzing": ("quiz",),
    "ragged": ("rag",),
    "ragging": ("rag",),
    "rallied": ("rally",),
    "ramified": ("ramify",),
    "rammed": ("ram",),
    "ramming": ("ram",),
    "ran": ("run",),
    "rang": ("ring",),
    "rapped": ("rap",),
    "rappelled": ("rappel",),
    "rappelling": ("rappel",),
    "rapping": ("rap",),
    "rarefied": ("rarefy",),
    "ratified": ("ratify",),
    "ratted": ("rat",),
    "ratting": ("rat",),
    "ravelled": ("ravel",),
    "ravelling": ("ravel",),
    "razor-cutting": ("razor-cut",),
    "re-trod": ("re-tread",),
    "re-trodden": ("re-tread",),
    "rebelled": ("rebel",),
    "rebelling": ("rebel",),
    "rebuilt": ("rebuild",),
    "rebutted": ("rebut",),
    "rebutting": ("rebut",),
    "recapped": ("recap",),
    "recapping": ("recap",),
    "reclassified": ("reclassify",),
    "recommitted": ("recommit",),
    "recommitting": ("recommit",),
    "recopied": ("recopy",),
    "rectified": ("rectify",),
    "recurred": ("recur",),
    "recurring": ("recur",),
    "red": ("red",),
    "red-pencilled": ("red-pencil",),
    "red-pencilling": ("red-pencil",),
    "redded": ("red", "redd",),
    "redding": ("red", "redd",),
    "redid": ("redo",),
    "redone": ("redo",),
    "referred": ("refer",),
    "referring": ("refer",),
    "refitted": ("refit",),
    "refitting": ("refit",),
    "reft": ("reave",),
    "refuelled": ("refuel",),
    "refuelling": ("refuel",),
    "regretted": ("regret",),
    "regretting": ("regret",),
    "reheard": ("rehear",),
    "reified": ("reify",),
    "relied": ("rely",),
    "remade": ("remake",),
    "remarried": ("remarry",),
    "remitted": ("remit",),
    "remitting": ("remit",),
    "rent": ("rend",),
    "repaid": ("repay",),
    "repelled": ("repel",),
    "repelling": ("repel",),
    "replevied": ("replevy",),
    "replied": ("reply",),
    "repotted": ("repot",),
    "repotting": ("repot",),
    "reran": ("rerun",),
    "rerunning": ("rerun",),
    "resat": ("resit",),
    "resetting": ("reset",),
    "resewn": ("resew",),
    "resitting": ("resit",),
    "retaken": ("retake",),
    "rethought": ("rethink",),
    "retold": ("retell",),
    "retook": ("retake",),
    "retransmitted": ("retransmit",),
    "retransmitting": ("retransmit",),
    "retried": ("retry",),
    "retrofitted": ("retrofit",),
    "retrofitting": ("retrofit",),
    "retted": ("ret",),
    "retting": ("ret",),
    "reunified": ("reunify",),
    "revelled": ("revel",),
    "revelling": ("revel",),
    "revetted": ("revet",),
    "revetting": ("revet",),
    "revivified": ("revivify",),
    "revved": ("rev",),
    "revving": ("rev",),
    "rewound": ("rewind",),
    "rewritten": ("rewrite",),
    "rewrote": ("rewrite",),
    "ribbed": ("rib",),
    "ribbing": ("rib",),
    "ricochetted": ("ricochet",),
    "ricochetting": ("ricochet",),
    "ridded": ("rid",),
    "ridden": ("ride",),
    "ridding": ("rid",),
    "rigged": ("rig",),
    "rigging": ("rig",),
    "rigidified": ("rigidify",),
    "rimmed": ("rim",),
    "rimming": ("rim",),
    "ripped": ("rip",),
    "ripping": ("rip",),
    "risen": ("rise",),
    "rivalled": ("rival",),
    "rivalling": ("rival",),
    "riven": ("rive",),
    "robbed": ("rob",),
    "robbing": ("rob",),
    "rode": ("ride",),
    "rose": ("rise",),
    "rotted": ("rot",),
    "rotting": ("rot",),
    "rough-dried": ("rough-dry",),
    "rough-hewn": ("rough-hew",),
    "rove": ("reeve",),
    "rowelled": ("rowel",),
    "rowelling": ("rowel",),
    "rubbed": ("rub",),
    "rubbing": ("rub",),
    "rung": ("ring",),
    "running": ("run",),
    "rutted": ("rut",),
    "rutting": ("rut",),
    "saccharified": ("saccharify",),
    "sagged": ("sag",),
    "sagging": ("sag",),
    "said": ("say",),
    "salaried": ("salary",),
    "salified": ("salify",),
    "sallied": ("sally",),
    "sanctified": ("sanctify",),
    "sandbagged": ("sandbag",),
    "sandbagging": ("sandbag",),
    "sang": ("sing",),
    "sank": ("sink",),
    "saponified": ("saponify",),
    "sapped": ("sap",),
    "sapping": ("sap",),
    "sat": ("sit",),
    "satisfied": ("satisfy",),
    "savvied": ("savvy",),
    "saw": ("see",),
    "sawn": ("saw",),
    "scagged": ("scag",),
    "scagging": ("scag",),
    "scanned": ("scan",),
    "scanning": ("scan",),
    "scarified": ("scarify",),
    "scarred": ("scar",),
    "scarring": ("scar",),
    "scatted": ("scat",),
    "scatting": ("scat",),
    "scorified": ("scorify",),
    "scragged": ("scrag",),
    "scragging": ("scrag",),
    "scrammed": ("scram",),
    "scramming": ("scram",),
    "scrapped": ("scrap",),
    "scrapping": ("scrap",),
    "scried": ("scry",),
    "scrubbed": ("scrub",),
    "scrubbing": ("scrub",),
    "scrummed": ("scrum",),
    "scrumming": ("scrum",),
    "scudded": ("scud",),
    "scudding": ("scud",),
    "scummed": ("scum",),
    "scumming": ("scum",),
    "scurried": ("scurry",),
    "seed": ("seed",),
    "seen": ("see",),
    "sent": ("send",),
    "setting": ("set",),
    "sewn": ("sew",),
    "shagged": ("shag",),
    "shagging": ("shag",),
    "shaken": ("shake",),
    "shaken_hands": ("shake_hands",),
    "shakes_hands": ("shake_hands",),
    "shaking_hands": ("shake_hands",),
    "shammed": ("sham",),
    "shamming": ("sham",),
    "sharecropped": ("sharecrop",),
    "sharecropping": ("sharecrop",),
    "shat": ("shit",),
    "shaven": ("shave",),
    "shed": ("shed",),
    "shedding": ("shed",),
    "shellacked": ("shellac",),
    "shellacking": ("shellac",),
    "shent": ("shend",),
    "shewn": ("shew",),
    "shied": ("shy",),
    "shikarred": ("shikar",),
    "shikarring": ("shikar",),
    "shillyshallied": ("shillyshally",),
    "shimmed": ("shim",),
    "shimmied": ("shimmy",),
    "shimming": ("shim",),
    "shinned": ("shin",),
    "shinning": ("shin",),
    "shipped": ("ship",),
    "shipping": ("ship",),
    "shitted": ("shit",),
    "shitting": ("shit",),
    "shod": ("shoe",),
    "shone": ("shine",),
    "shook": ("shake",),
    "shook_hands": ("shake_hands",),
    "shopped": ("shop",),
    "shopping": ("shop",),
    "shot": ("shoot",),
    "shotgunned": ("shotgun",),
    "shotgunning": ("shotgun",),
    "shotted": ("shot",),
    "shotting": ("shot",),
    "shovelled": ("shovel",),
    "shovelling": ("shovel",),
    "shown": ("show",),
    "shrank": ("shrink",),
    "shredded": ("shred",),
    "shredding": ("shred",),
    "shrink-wrapped": ("shrink-wrap",),
    "shrink-wrapping": ("shrink-wrap",),
    "shrivelled": ("shrivel",),
    "shrivelling": ("shrivel",),
    "shriven": ("shrive",),
    "shrove": ("shrive",),
    "shrugged": ("shrug",),
    "shrugging": ("shrug",),
    "shrunk": ("shrink",),
    "shrunken": ("shrink",),
    "shunned": ("shun",),
    "shunning": ("shun",),
    "shutting": ("shut",),
    "sicked": ("sic",),
    "sicking": ("sic",),
    "sideslipped": ("sideslip",),
    "sideslipping": ("sideslip",),
    "sidestepped": ("sidestep",),
    "sidestepping": ("sidestep",),
    "sightsaw": ("sightsee",),
    "sightseen": ("sightsee",),
    "signalled": ("signal",),
    "signalling": ("signal",),
    "signified": ("signify",),
    "silicified": ("silicify",),
    "simplified": ("simplify",),
    "singing": ("sing", "singe",),
    "single-stepped": ("single-step",),
    "single-stepping": ("single-step",),
    "sinned": ("sin",),
    "sinning": ("sin",),
    "sipped": ("sip",),
    "sipping": ("sip",),
    "sitting": ("sit",),
    "skellied": ("skelly",),
    "skenned": ("sken",),
    "skenning": ("sken",),
    "sketted": ("sket",),
    "sketting": ("sket",),
    "ski'd": ("ski",),
    "skidded": ("skid",),
    "skidding": ("skid",),
    "skimmed": ("skim",),
    "skimming": ("skim",),
    "skin-popped": ("skin-pop",),
    "skin-popping": ("skin-pop",),
    "skinned": ("skin",),
    "skinning": ("skin",),
    "skinny-dipped": ("skinny-dip",),
    "skinny-dipping": ("skinny-dip",),
    "skipped": ("skip",),
    "skipping": ("skip",),
    "skivvied": ("skivvy",),
    "skydove": ("skydive",),
    "slabbed": ("slab",),
    "slabbing": ("slab",),
    "slagged": ("slag",),
    "slagging": ("slag",),
    "slain": ("slay",),
    "slammed": ("slam",),
    "slamming": ("slam",),
    "slapped": ("slap",),
    "slapping": ("slap",),
    "slatted": ("slat",),
    "slatting": ("slat",),
    "sledding": ("sled",),
    "slept": ("sleep",),
    "slew": ("slay",),
    "slid": ("slide",),
    "slidden": ("slide",),
    "slipped": ("slip",),
    "slipping": ("slip",),
    "slitting": ("slit",),
    "slogged": ("slog",),
    "slogging": ("slog",),
    "slopped": ("slop",),
    "slopping": ("slop",),
    "slotted": ("slot",),
    "slotting": ("slot",),
    "slugged": ("slug",),
    "slugging": ("slug",),
    "slummed": ("slum",),
    "slumming": ("slum",),
    "slung": ("sling",),
    "slunk": ("slink",),
    "slurred": ("slur",),
    "slurring": ("slur",),
    "smelt": ("smell",),
    "smit": ("smite",),
    "smitten": ("smite",),
    "smote": ("smite",),
    "smutted": ("smut",),
    "smutting": ("smut",),
    "snagged": ("snag",),
    "snagging": ("snag",),
    "snapped": ("snap",),
    "snapping": ("snap",),
    "snedded": ("sned",),
    "snedding": ("sned",),
    "snipped": ("snip",),
    "snipping": ("snip",),
    "snivelled": ("snivel",),
    "snivelling": ("snivel",),
    "snogged": ("snog",),
    "snogging": ("snog",),
    "snubbed": ("snub",),
    "snubbing": ("snub",),
    "snuck": ("sneak",),
    "snugged": ("snug",),
    "snugging": ("snug",),
    "sobbed": ("sob",),
    "sobbing": ("sob",),
    "sodded": ("sod",),
    "sodding": ("sod",),
    "soft-pedalled": ("soft-pedal",),
    "soft-pedalling": ("soft-pedal",),
    "sold": ("sell",),
    "solemnified": ("solemnify",),
    "solidified": ("solidify",),
    "soothsaid": ("soothsay",),
    "sopped": ("sop",),
    "sopping": ("sop",),
    "sought": ("seek",),
    "sown": ("sow",),
    "spagged": ("spag",),
    "spagging": ("spag",),
    "spancelled": ("spancel",),
    "spancelling": ("spancel",),
    "spanned": ("span",),
    "spanning": ("span",),
    "sparred": ("spar",),
    "sparring": ("spar",),
    "spat": ("spit",),
    "spatted": ("spat",),
    "spatting": ("spat",),
    "specified": ("specify",),
    "sped": ("speed",),
    "speechified": ("speechify",),
    "spellbound": ("spellbind",),
    "spelt": ("spell",),
    "spent": ("spend",),
    "spied": ("spy",),
    "spilt": ("spill",),
    "spin-dried": ("spin-dry",),
    "spinning": ("spin",),
    "spiralled": ("spiral",),
    "spiralling": ("spiral",),
    "spitted": ("spit",),
    "spitting": ("spit",),
    "splitting": ("split",),
    "spoilt": ("spoil",),
    "spoke": ("speak",),
    "spoken": ("speak",),
    "spoon-fed": ("spoon-feed",),
    "spotlit": ("spotlight",),
    "spotted": ("spot",),
    "spotting": ("spot",),
    "sprang": ("spring",),
    "sprigged": ("sprig",),
    "sprigging": ("sprig",),
    "sprung": ("spring",),
    "spudded": ("spud",),
    "spudding": ("spud",),
    "spun": ("spin",),
    "spurred": ("spur",),
    "spurring": ("spur",),
    "squatted": ("squat",),
    "squatting": ("squat",),
    "squibbed": ("squib",),
    "squibbing": ("squib",),
    "squidded": ("squid",),
    "squidding": ("squid",),
    "squilgee": ("squeegee",),
    "stabbed": ("stab",),
    "stabbing": ("stab",),
    "stall-fed": ("stall-feed",),
    "stank": ("stink",),
    "starred": ("star",),
    "starring": ("star",),
    "steadied": ("steady",),
    "stellified": ("stellify",),
    "stemmed": ("stem",),
    "stemming": ("stem",),
    "stems_from": ("stem_from",),
    "stencilled": ("stencil",),
    "stencilling": ("stencil",),
    "stepped": ("step",),
    "stepping": ("step",),
    "stetted": ("stet",),
    "stetting": ("stet",),
    "stied": ("sty",),
    "stilettoeing": ("stiletto",),
    "stirred": ("stir",),
    "stirring": ("stir",),
    "stole": ("steal",),
    "stolen": ("steal",),
    "stood": ("stand",),
    "stopped": ("stop",),
    "stopping": ("stop",),
    "storied": ("story",),
    "stotted": ("stot",),
    "stotting": ("stot",),
    "stove": ("stave",),
    "strapped": ("strap",),
    "strapping": ("strap",),
    "stratified": ("stratify",),
    "strewn": ("strew",),
    "stridden": ("stride",),
    "stripped": ("strip",),
    "stripping": ("strip",),
    "striven": ("strive",),
    "strode": ("stride",),
    "stropped": ("strop",),
    "stropping": ("strop",),
    "strove": ("strive",),
    "strown": ("strow",),
    "struck": ("strike",),
    "strummed": ("strum",),
    "strumming": ("strum",),
    "strung": ("string",),
    "strutted": ("strut",),
    "strutting": ("strut",),
    "stubbed": ("stub",),
    "stubbing": ("stub",),
    "stuck": ("stick",),
    "studded": ("stud",),
    "studding": ("stud",),
    "studied": ("study",),
    "stultified": ("stultify",),
    "stummed": ("stum",),
    "stumming": ("stum",),
    "stung": ("sting",),
    "stunk": ("stink",),
    "stunned": ("stun",),
    "stunning": ("stun",),
    "stupefied": ("stupefy",),
    "stymying": ("stymie",),
    "subbed": ("sub",),
    "subbing": ("sub",),
    "subjectified": ("subjectify",),
    "subletting": ("sublet",),
    "submitted": ("submit",),
    "submitting": ("submit",),
    "subtotalled": ("subtotal",),
    "subtotalling": ("subtotal",),
    "sullied": ("sully",),
    "sulphuretted": ("sulphuret",),
    "sulphuretting": ("sulphuret",),
    "summed": ("sum",),
    "summing": ("sum",),
    "sung": ("sing",),
    "sunk": ("sink",),
    "sunken": ("sink",),
    "sunned": ("sun",),
    "sunning": ("sun",),
    "supped": ("sup",),
    "supping": ("sup",),
    "supplied": ("supply",),
    "swabbed": ("swab",),
    "swabbing": ("swab",),
    "swagged": ("swag",),
    "swagging": ("swag",),
    "swam": ("swim",),
    "swapped": ("swap",),
    "swapping": ("swap",),
    "swatted": ("swat",),
    "swatting": ("swat",),
    "swept": ("sweep",),
    "swigged": ("swig",),
    "swigging": ("swig",),
    "swimming": ("swim",),
    "swivelled": ("swivel",),
    "swivelling": ("swivel",),
    "swollen": ("swell",),
    "swopped": ("swap",),
    "swopping": ("swap",),
    "swops": ("swap",),
    "swore": ("swear",),
    "sworn": ("swear",),
    "swotted": ("swot",),
    "swotting": ("swot",),
    "swum": ("swim",),
    "swung": ("swing",),
    "syllabified": ("syllabify",),
    "symbolled": ("symbol",),
    "symbolling": ("symbol",),
    "tabbed": ("tab",),
    "tabbing": ("tab",),
    "tagged": ("tag",),
    "tagging": ("tag",),
    "taken": ("take",),
    "taken_a_side": ("take_a_side",),
    "taken_pains": ("take_pains",),
    "taken_steps": ("take_steps",),
    "takes_a_side": ("take_a_side",),
    "takes_pains": ("take_pains",),
    "takes_steps": ("take_steps",),
    "taking_a_side": ("take_a_side",),
    "taking_pains": ("take_pains",),
    "taking_steps": ("take_steps",),
    "talcked": ("talc",),
    "talcking": ("talc",),
    "tallied": ("tally",),
    "tally-ho'd": ("tally-ho",),
    "tammied": ("tammy",),
    "tanned": ("tan",),
    "tanning": ("tan",),
    "tapped": ("tap",),
    "tapping": ("tap",),
    "tarred": ("tar",),
    "tarried": ("tarry",),
    "tarring": ("tar",),
    "tasselled": ("tassel",),
    "tasselling": ("tassel",),
    "tatted": ("tat",),
    "tatting": ("tat",),
    "taught": ("teach",),
    "taxis": ("taxis",),
    "taxying": ("taxi",),
    "teaselled": ("teasel",),
    "teaselling": ("teasel",),
    "tedded": ("ted",),
    "tedding": ("ted",),
    "tepefied": ("tepefy",),
    "terrified": ("terrify",),
    "testes": ("testes",),
    "testified": ("testify",),
    "thinking_the_world_of": ("think_the_world_of",),
    "thinks_the_world_of": ("think_the_world_of",),
    "thinned": ("thin",),
    "thinning": ("thin",),
    "thought": ("think",),
    "thought_the_world_of": ("think_the_world_of",),
    "threw": ("throw",),
    "threw_out": ("throw_out",),
    "thriven": ("thrive",),
    "throbbed": ("throb",),
    "throbbing": ("throb",),
    "throve": ("thrive",),
    "throwing_out": ("throw_out",),
    "thrown": ("throw",),
    "thrown_out": ("throw_out",),
    "throws_out": ("throw_out",),
    "thrummed": ("thrum",),
    "thrumming": ("thrum",),
    "thudded": ("thud",),
    "thudding": ("thud",),
    "tidied": ("tidy",),
    "tinned": ("tin",),
    "tinning": ("tin",),
    "tinselled": ("tinsel",),
    "tinselling": ("tinsel",),
    "tipped": ("tip",),
    "tipping": ("tip",),
    "tittupped": ("tittup",),
    "tittupping": ("tittup",),
    "toadied": ("toady",),
    "togged": ("tog",),
    "togging": ("tog",),
    "told": ("tell",),
    "took": ("take",),
    "took_a_side": ("take_a_side",),
    "took_pains": ("take_pains",),
    "took_steps": ("take_steps",),
    "topped": ("top",),
    "topping": ("top",),
    "tore": ("tear",),
    "torn": ("tear",),
    "torrefied": ("torrefy",),
    "torrify": ("torrefy",),
    "totalled": ("total",),
    "totalling": ("total",),
    "totted": ("tot",),
    "totting": ("tot",),
    "towelled": ("towel",),
    "towelling": ("towel",),
    "trafficked": ("traffic",),
    "trafficking": ("traffic",),
    "trameled": ("trammel",),
    "trameling": ("trammel",),
    "tramelled": ("trammel",),
    "tramelling": ("trammel",),
    "tramels": ("trammel",),
    "trammed": ("tram",),
    "tramming": ("tram",),
    "transferred": ("transfer",),
    "transferring": ("transfer",),
    "transfixt": ("transfix",),
    "tranship": ("transship",),
    "transhipped": ("tranship",),
    "transhipping": ("tranship",),
    "transmitted": ("transmit",),
    "transmitting": ("transmit",),
    "transmogrified": ("transmogrify",),
    "transshipped": ("transship",),
    "transshipping": ("transship",),
    "trapanned": ("trapan",),
    "trapanning": ("trapan",),
    "trapped": ("trap",),
    "trapping": ("trap",),
    "travelled": ("travel",),
    "travelling": ("travel",),
    "travestied": ("travesty",),
    "trekked": ("trek",),
    "trekking": ("trek",),
    "trepanned": ("trepan",),
    "trepanning": ("trepan",),
    "tried": ("try",),
    "trigged": ("trig",),
    "trigging": ("trig",),
    "trimmed": ("trim",),
    "trimming": ("trim",),
    "tripped": ("trip",),
    "tripping": ("trip",),
    "trod": ("tread",),
    "trodden": ("tread",),
    "trogged": ("trog",),
    "trogging": ("trog",),
    "trotted": ("trot",),
    "trotting": ("trot",),
    "trowelled": ("trowel",),
    "trowelling": ("trowel",),
    "tugged": ("tug",),
    "tugging": ("tug",),
    "tumefied": ("tumefy",),
    "tunned": ("tun",),
    "tunnelled": ("tunnel",),
    "tunnelling": ("tunnel",),
    "tunning": ("tun",),
    "tupped": ("tup",),
    "tupping": ("tup",),
    "tut-tutted": ("tut-tut",),
    "tut-tutting": ("tut-tut",),
    "twigged": ("twig",),
    "twigging": ("twig",),
    "twinned": ("twin",),
    "twinning": ("twin",),
    "twitted": ("twit",),
    "twitting": ("twit",),
    "tying": ("tie",),
    "typesetting": ("typeset",),
    "typewritten": ("typewrite",),
    "typewrote": ("typewrite",),
    "typified": ("typify",),
    "uglified": ("uglify",),
    "unbarred": ("unbar",),
    "unbarring": ("unbar",),
    "unbent": ("unbend",),
    "unbound": ("unbind",),
    "uncapped": ("uncap",),
    "uncapping": ("uncap",),
    "unclad": ("unclothe",),
    "unclogged": ("unclog",),
    "unclogging": ("unclog",),
    "underbidding": ("underbid",),
    "underbought": ("underbuy",),
    "undercutting": ("undercut",),
    "underfed": ("underfeed",),
    "undergirt": ("undergird",),
    "undergone": ("undergo",),
    "underlaid": ("underlay",),
    "underlain": ("underlie",),
    "underlay": ("underlie",),
    "underletting": ("underlet",),
    "underlying": ("underlie",),
    "underpaid": ("underpay",),
    "underpinned": ("underpin",),
    "underpinning": ("underpin",),
    "underpropped": ("underprop",),
    "underpropping": ("underprop",),
    "undersetting": ("underset",),
    "undershot": ("undershoot",),
    "undersold": ("undersell",),
    "understood": ("understand",),
    "understudied": ("understudy",),
    "undertaken": ("undertake",),
    "undertook": ("undertake",),
    "underwent": ("undergo",),
    "underwritten": ("underwrite",),
    "underwrote": ("underwrite",),
    "undid": ("undo",),
    "undone": ("undo",),
    "unfitted": ("unfit",),
    "unfitting": ("unfit",),
    "unfroze": ("unfreeze",),
    "unfrozen": ("unfreeze",),
    "unified": ("unify",),
    "unkennelled": ("unkennel",),
    "unkennelling": ("unkennel",),
    "unknitted": ("unknit",),
    "unknitting": ("unknit",),
    "unlaid": ("unlay",),
    "unlearnt": ("unlearn",),
    "unmade": ("unmake",),
    "unmanned": ("unman",),
    "unmanning": ("unman",),
    "unpegged": ("unpeg",),
    "unpegging": ("unpeg",),
    "unpinned": ("unpin",),
    "unpinning": ("unpin",),
    "unplugged": ("unplug",),
    "unplugging": ("unplug",),
    "unravelled": ("unravel",),
    "unravelling": ("unravel",),
    "unrigged": ("unrig",),
    "unrigging": ("unrig",),
    "unripped": ("unrip",),
    "unripping": ("unrip",),
    "unrove": ("unreeve",),
    "unsaid": ("unsay",),
    "unshipped": ("unship",),
    "unshipping": ("unship",),
    "unslung": ("unsling",),
    "unsnapped": ("unsnap",),
    "unsnapping": ("unsnap",),
    "unspoke": ("unspeak",),
    "unspoken": ("unspeak",),
    "unsteadied": ("unsteady",),
    "unstepped": ("unstep",),
    "unstepping": ("unstep",),
    "unstopped": ("unstop",),
    "unstopping": ("unstop",),
    "unstrung": ("unstring",),
    "unstuck": ("unstick",),
    "unswore": ("unswear",),
    "unsworn": ("unswear",),
    "untaught": ("unteach",),
    "unthought": ("unthink",),
    "untidied": ("untidy",),
    "untrod": ("untread",),
    "untrodden": ("untread",),
    "untying": ("untie",),
    "unwound": ("unwind",),
    "unwrapped": ("unwrap",),
    "unwrapping": ("unwrap",),
    "unzipped": ("unzip",),
    "unzipping": ("unzip",),
    "upbuilt": ("upbuild",),
    "upheld": ("uphold",),
    "uphove": ("upheave",),
    "upped": ("up",),
    "uppercutting": ("uppercut",),
    "upping": ("up",),
    "uprisen": ("uprise",),
    "uprose": ("uprise",),
    "upsetting": ("upset",),
    "upsprang": ("upspring",),
    "upsprung": ("upspring",),
    "upswept": ("upsweep",),
    "upswollen": ("upswell",),
    "upswung": ("upswing",),
    "vagged": ("vag",),
    "vagging": ("vag",),
    "varied": ("vary",),
    "vatted": ("vat",),
    "vatting": ("vat",),
    "verbified": ("verbify",),
    "verified": ("verify",),
    "versified": ("versify",),
    "vetted": ("vet",),
    "vetting": ("vet",),
    "victualled": ("victual",),
    "victualling": ("victual",),
    "vilified": ("vilify",),
    "vitrified": ("vitrify",),
    "vitriolled": ("vitriol",),
    "vitriolling": ("vitriol",),
    "vivified": ("vivify",),
    "vying": ("vie",),
    "wadded": ("wad",),
    "waddied": ("waddy",),
    "wadding": ("wad",),
    "wadsetted": ("wadset",),
    "wadsetting": ("wadset",),
    "wagged": ("wag",),
    "wagging": ("wag",),
    "wanned": ("wan",),
    "wanning": ("wan",),
    "warred": ("war",),
    "warring": ("war",),
    "was": ("be",),
    "water-ski'd": ("water-ski",),
    "waylaid": ("waylay",),
    "wearied": ("weary",),
    "weatherstripped": ("weatherstrip",),
    "weatherstripping": ("weatherstrip",),
    "webbed": ("web",),
    "webbing": ("web",),
    "wedded": ("wed",),
    "wedding": ("wed",),
    "weed": ("weed",),
    "went": ("go",),
    "went_deep": ("go_deep",),
    "wept": ("weep",),
    "were": ("be",),
    "wetted": ("wet",),
    "wetting": ("wet",),
    "whammed": ("wham",),
    "whamming": ("wham",),
    "whapped": ("whap",),
    "whapping": ("whap",),
    "whetted": ("whet",),
    "whetting": ("whet",),
    "whinnied": ("whinny",),
    "whipped": ("whip",),
    "whipping": ("whip",),
    "whipsawn": ("whipsaw",),
    "whirred": ("whir",),
    "whirring": ("whir",),
    "whistle-stopped": ("whistle-stop",),
    "whistle-stopping": ("whistle-stop",),
    "whizzed": ("whiz",),
    "whizzes": ("whiz",),
    "whizzing": ("whiz",),
    "whopped": ("whop",),
    "whopping": ("whop",),
    "wigged": ("wig",),
    "wigging": ("wig",),
    "wigwagged": ("wigwag",),
    "wigwagging": ("wigwag",),
    "wildcatted": ("wildcat",),
    "wildcatting": ("wildcat",),
    "window-shopped": ("window-shop",),
    "window-shopping": ("window-shop",),
    "winning": ("win",),
    "winterfed": ("winterfeed",),
    "wiredrawn": ("wiredraw",),
    "wiredrew": ("wiredraw",),
    "withdrawn": ("withdraw",),
    "withdrew": ("withdraw",),
    "withheld": ("withhold",),
    "withstood": ("withstand",),
    "woke": ("wake",),
    "woken": ("wake",),
    "won": ("win",),
    "wonned": ("won",),
    "wonning": ("won",),
    "wore": ("wear",),
    "worn": ("wear",),
    "worried": ("worry",),
    "worshipped": ("worship",),
    "worshipping": ("worship",),
    "wound": ("wind",),
    "wove": ("weave",),
    "woven": ("weave",),
    "wrapped": ("wrap",),
    "wrapping": ("wrap",),
    "wried": ("wry",),
    "written": ("write",),
    "wrote": ("write",),
    "wrought": ("work",),
    "wrung": ("wring",),
    "yakked": ("yak",),
    "yakking": ("yak",),
    "yapped": ("yap",),
    "yapping": ("yap",),
    "ycleped": ("clepe",),
    "yclept": ("clepe",),
    "yenned": ("yen",),
    "yenning": ("yen",),
    "yodelled": ("yodel",),
    "yodelling": ("yodel",),
    "zapped": ("zap",),
    "zapping": ("zap",),
    "zigzagged": ("zigzag",),
    "zigzagging": ("zigzag",),
    "zipped": ("zip",),
    "zipping": ("zip",)
}



# coding: utf8
from __future__ import unicode_literals, print_function

import requests
import pkg_resources
from pathlib import Path
import sys

from ..compat import path2str, locale_escape
from ..util import prints, get_data_path, read_json
from .. import about


def validate():
    """Validate that the currently installed version of spaCy is compatible
    with the installed models. Should be run after `pip install -U spacy`.
    """
    r = requests.get(about.__compatibility__)
    if r.status_code != 200:
        prints("Couldn't fetch compatibility table.",
               title="Server error (%d)" % r.status_code, exits=1)
    compat = r.json()['spacy']
    current_compat = compat.get(about.__version__)
    if not current_compat:
        prints(about.__compatibility__, exits=1,
               title="Can't find spaCy v{} in compatibility table"
               .format(about.__version__))
    all_models = set()
    for spacy_v, models in dict(compat).items():
        all_models.update(models.keys())
        for model, model_vs in models.items():
            compat[spacy_v][model] = [reformat_version(v) for v in model_vs]
    model_links = get_model_links(current_compat)
    model_pkgs = get_model_pkgs(current_compat, all_models)
    incompat_links = {l for l, d in model_links.items() if not d['compat']}
    incompat_models = {d['name'] for _, d in model_pkgs.items()
                       if not d['compat']}
    incompat_models.update([d['name'] for _, d in model_links.items()
                            if not d['compat']])
    na_models = [m for m in incompat_models if m not in current_compat]
    update_models = [m for m in incompat_models if m in current_compat]

    prints(path2str(Path(__file__).parent.parent),
           title="Installed models (spaCy v{})".format(about.__version__))
    if model_links or model_pkgs:
        print(get_row('TYPE', 'NAME', 'MODEL', 'VERSION', ''))
        for name, data in model_pkgs.items():
            print(get_model_row(current_compat, name, data, 'package'))
        for name, data in model_links.items():
            print(get_model_row(current_compat, name, data, 'link'))
    else:
        prints("No models found in your current environment.", exits=0)

    if update_models:
        cmd = '    python -m spacy download {}'
        print("\n    Use the following commands to update the model packages:")
        print('\n'.join([cmd.format(pkg) for pkg in update_models]))

    if na_models:
        prints("The following models are not available for spaCy v{}: {}"
               .format(about.__version__, ', '.join(na_models)))

    if incompat_links:
        prints("You may also want to overwrite the incompatible links using "
               "the `python -m spacy link` command with `--force`, or remove "
               "them from the data directory. Data path: {}"
               .format(path2str(get_data_path())))

    if incompat_models or incompat_links:
        sys.exit(1)


def get_model_links(compat):
    links = {}
    data_path = get_data_path()
    if data_path:
        models = [p for p in data_path.iterdir() if is_model_path(p)]
        for model in models:
            meta_path = Path(model) / 'meta.json'
            if not meta_path.exists():
                continue
            meta = read_json(meta_path)
            link = model.parts[-1]
            name = meta['lang'] + '_' + meta['name']
            links[link] = {'name': name, 'version': meta['version'],
                           'compat': is_compat(compat, name, meta['version'])}
    return links


def get_model_pkgs(compat, all_models):
    pkgs = {}
    for pkg_name, pkg_data in pkg_resources.working_set.by_key.items():
        package = pkg_name.replace('-', '_')
        if package in all_models:
            version = pkg_data.version
            pkgs[pkg_name] = {'name': package, 'version': version,
                              'compat': is_compat(compat, package, version)}
    return pkgs


def get_model_row(compat, name, data, type='package'):
    tpl_red = '\x1b[38;5;1m{}\x1b[0m'
    tpl_green = '\x1b[38;5;2m{}\x1b[0m'
    if data['compat']:
        comp = tpl_green.format(locale_escape('✔', errors='ignore'))
        version = tpl_green.format(data['version'])
    else:
        comp = '--> {}'.format(compat.get(data['name'], ['n/a'])[0])
        version = tpl_red.format(data['version'])
    return get_row(type, name, data['name'], version, comp)


def get_row(*args):
    tpl_row = '    {:<10}' + ('  {:<20}' * 4)
    return tpl_row.format(*args)


def is_model_path(model_path):
    exclude = ['cache', 'pycache', '__pycache__']
    name = model_path.parts[-1]
    return (model_path.is_dir() and name not in exclude
            and not name.startswith('.'))


def is_compat(compat, name, version):
    return name in compat and version in compat[name]


def reformat_version(version):
    """Hack to reformat old versions ending on '-alpha' to match pip format."""
    if version.endswith('-alpha'):
        return version.replace('-alpha', 'a0')
    return version.replace('-alpha', 'a')
